%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% diagnostic.m	Script to plot the output of the quasi-geostrophic annulus model.
%
% pdw 15Aug02 -- 12Mar03
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% hold on; r_circ = 0.0625; theta_circ = 0:0.01:2*pi; plot(r_circ*cos(theta_circ), r_circ*sin(theta_circ))
% colormap('gray')

close all

again = 1 ;
while (again ~= 0)

% housekeeping
clear all
clear global

% declaration of global variables
global N_rad N_azim N_layer machineformat precision

% data file definition
machineformat = 'ieee-le' ;		% 'ieee-le' for little endian format, 'ieee-be' for big endian format
precision = 'float32' ;			% 'float64' if data files are double precision, 'float32' if single precision

% repeat flag
again = 1 ;

% load in scalar run parameters from qgam_data.m
qgam_data ;

% clear screen
clc ;

% define vector run parameters: these lines need to be manually edited if the fluid densities or kinematic viscosities are altered
rho(1) = 997.0 ;  
rho(2) = 1003.0 ; 
new(1) = 1.27e-6 ;
new(2) = 1.08e-6 ;

% calculation of physical parameters
f = 2*omega ;
delta_rho = rho(2) - rho(1) ;
rho_bar = (rho(2) + rho(1))/2.0 ; 
g_primed = g*delta_rho/rho_bar ;
chi = sqrt(new(2)/new(1)) ;
delta_omega(1) = 0.5*(2.0 + chi)*lid_delta_omega/(1.0 + chi) ;
delta_omega(2) = 0.5*lid_delta_omega/(1.0 + chi) ;
delta_m = sqrt(S/(g*delta_rho)) ;
F = f*f*(b - a)*(b - a)/(g_primed*H) ;
d = sqrt(omega*0.5*(new(1) + new(2)))/(H*lid_delta_omega) ;

% calculation of grid parameters
delta_r = (b - a)/(N_rad - 1) ;				% scalar
delta_theta = 2*pi/N_azim ;				% scalar
r = [a : delta_r : b] ;					% vector of size (N_rad)
theta = [0 : delta_theta : 2*pi - 2*pi/N_azim] ;	% vector of size (N_azim)
[theta_mat, r_mat] = meshgrid(theta, r) ;		% both 2D arrays of size (N_rad, N_azim)
[x_mat, y_mat] = pol2cart(theta_mat, r_mat) ;		% both 2D arrays of size (N_rad, N_azim)
% element (i, j) of the read-in arrays has radius = r_mat(i, j), angle = theta_mat(i, j), x = x_mat(i, j) and y = y_mat(i, j)

% get working directory
wd = pwd ;
remove = '/a/ahab/home/ahab/gfd3/williams/QUAGMIRE' ;		% remove unwanted long prefix
if wd(1:length(remove))==remove ;
	wd = wd(length(remove)+1:length(wd)) ;
end ;
for i = 1:length(wd) ;			% replace "_" with "-" because underscores are interpreted as subscript commands by Matlab
	if wd(i) == '_' ;
		wd(i) = '-' ;
	end ;
end ;

% set font size for titles, axis labels, axis numbers
set(0, 'DefaultAxesFontSize', 15) ;	% default is 10;  use 20 for printouts which will be halved b4 insertion into documents
set(0, 'DefaultTextFontSize', 15) ;	% ""
set(0, 'DefaultAxesFontName', 'Times') ;

% define matrix containing relative areas associated with each grid point, for use when evaluating volume integrals
area_factor = ones(N_rad,N_azim) ;
area_factor(1,:)=0.5 ;
area_factor(N_rad,:)=0.5 ;

% ask user what to plot
fprintf(1, '\n                  QUAGMIRE diagnostics package\n')
fprintf(1, '                  ^^^^^^^^^^^^^^^^^^^^^^^^^^^^\n\n')
fprintf(1, ['     current directory is ', wd, '\n\n'])
fprintf(1, '     0 = exit\n')
fprintf(1, '     1 = contour plot of perturbation streamfunction\n')
fprintf(1, '     2 = contour plot of perturbation potential vorticity\n')
fprintf(1, '     3*= contour plot of perturbation interface height\n')
fprintf(1, '     4 = vector plot of perturbation velocity field\n')
fprintf(1, '     5 = contour plot of full streamfunction\n')
fprintf(1, '     6 = contour plot of full potential vorticity\n')
fprintf(1, '     7 = contour plot of full interface height\n')
fprintf(1, '     8 = vector plot of full velocity field\n')
fprintf(1, '     9*= time-series of mid-radius interface height wave amplitude\n')
fprintf(1, '    10 = multiple cartesian diagnostics\n')
fprintf(1, '    11 = plot of gridpoint positions\n')
fprintf(1, '    12*= time-series of mid-radius interface height\n')
fprintf(1, '    13 = option 1 over-plotted with option 4\n')
fprintf(1, '    14 = option 5 over-plotted with option 8\n')
fprintf(1, '    15 = time-series of radially-averaged zonal perturbation velocity\n')
fprintf(1, '    16*= mid-radius perturbation potential vorticity Hovmueller diagram\n')
fprintf(1, '    17 = mid-radius perturbation streamfunction Fourier spectrum\n')
fprintf(1, '    18*= contour plot of Lighthill Radiation Term magnitude\n')
fprintf(1, '    19*= contour plot of Brown''s CAT indicator (phi)\n')
fprintf(1, '    20*= vector plot of full velocity shear field\n')
fprintf(1, '    21 = time-series of 1/4 and 3/4-radius interface height wave amplitude\n')
fprintf(1, '    22*= contour plot of local Richardson number in interfacial Ekman layer\n')
fprintf(1, '    23 = time series of minimum local Richardson number in interfacial Ekman layer\n')
fprintf(1, '    24*= contour plot of turbulent energy dissipation rate (epsilon)\n')
fprintf(1, '    25 = interface height profiles as a function of theta\n')
fprintf(1, '    26 = time series of system energy\n')
fprintf(1, '    27 = 3-D image of full interface height\n')
fprintf(1, '    28 = multiple radiation diagnostics\n')
fprintf(1, '    29 = contour plot of horizontal divergence\n')
fprintf(1, '    30 = mid-radius interface height Hovmueller diagram\n')
fprintf(1, '    31 = zonally-averaged zonal velocity as a function of radius\n')
fprintf(1, '    32 = time-radius Hovmueller plot of zonally-averaged zonal velocity\n')
option = input('\n     Which option? ') ;

if (option > 32) | (option < 0)
	fprintf(1, '     ERROR!  This is not a valid option.\n\n') ;
	return
end	

if (option ~= 0)
	close all
end

if (option == 1) | (option == 2) | (option == 4) | (option == 5) | (option == 6) | (option == 8) | (option == 10) | ...
		(option == 13) | (option == 14) | (option == 16) | (option == 19) | (option == 24) | (option == 28) | ...
		(option == 29) | (option == 31) | (option == 32)
	layer = input('     which layer? ') ;
	if (layer < 1) | (layer > N_layer)
		fprintf(1, '     ERROR!  This is not a valid layer number.\n\n') ;
		return
	end
end

if (option ~= 0) & (option ~= 11)
	start = input(['     start step? [eg. ', num2str(dump_period), '] ']) ;
	increment = input(['     increment? [eg. ', num2str(dump_period), '] ']) ;
	finish = input(['     final step? [eg. ', num2str(end_step), '] ']) ;
	if (option ~= 9) & (option ~= 10) & (option ~= 12) & (option ~= 15) & (option ~= 16) & ...
				(option ~= 21) & (option ~= 23) & (option ~= 26) & (option ~= 28) & (option ~= 30) & (option ~= 32)
		iteration = 0 ;
		if (start ~= finish)
			iteration = input('     number of movie iterations? ') ;
		end
		if (iteration ~= 0)
			fps = input(['     number of movie frames per second? [eg. ', num2str(1.0/(delta_t*increment)), ' for real time] ']) ;
		end
	end
	inds = [start : increment : finish] ;		% construct names of data files to be read in and plotted
	tmp_QGPV = ['QGPV.0000000000'] ;
	tmp_PSI_ = ['PSI_.0000000000'] ;   
	for i = 1 : length(inds)
		tmp = num2str(inds(i)) ;
		tmp_QGPV(16-length(tmp) : 15) = tmp ;
		tmp_PSI_(16-length(tmp) : 15) = tmp ;
		q_filenames(i, :) = tmp_QGPV ;
		psi_filenames(i, :) = tmp_PSI_ ;
	end
end

if (option == 9) | (option == 18) | (option == 19) | (option == 24) | (option == 25) | (option == 28) | (option == 29)
	windowSize = input('     azimuthal window size (in gridpoints, odd) for filtering? [1 for no filtering] ') ;
end

fprintf(1, '\n') ;

if option == 0 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

	% exit

	again = 0 ;

elseif option == 1 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

	% contour plot of perturbation streamfunction

	x_mat(:, N_azim+1) = x_mat(:, 1) ;				% } to join up data across final azimuthal gridpoint
	y_mat(:, N_azim+1) = y_mat(:, 1) ;				% }

	for i = 1 : length(inds)

		psi_primed = diagnostic_read(psi_filenames(i, :)) ;

		psi_primed(:, N_azim+1, :) = psi_primed(:, 1, :) ;	% to join up data across final azimuthal gridpoint
		contourf(x_mat(:,:), y_mat(:,:), psi_primed(:, :, layer), 20) ;
		shading flat
		colorbar_handle = colorbar ;
		set(get(colorbar_handle,'YLabel'),'String', ...
			['\psi '' (r, \theta, layer=', num2str(layer), ', t=', num2str(inds(i)*delta_t), 's)   (m^2 s^-^1)']) ;
		title([wd, '   (F, d) = (', num2str(F), ', ', num2str(d), ')']) ;
		axis([-1.1*b, 1.1*b, -1.1*b, 1.1*b]) ;
		xlabel('x (m)') ;
		ylabel('y (m)') ;
		axis equal 
		grid on 
		pause(0)
		if (iteration ~= 0)
			M(i) = getframe;
		end

	end

	if (iteration ~= 0)
		movie(M, iteration, fps)
	end

elseif option == 2 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

	% contour plot of perturbation potential vorticity

	x_mat(:, N_azim+1) = x_mat(:, 1) ;				% } to join up data across final azimuthal gridpoint
	y_mat(:, N_azim+1) = y_mat(:, 1) ;				% }

        for i = 1 : length(inds)

		q_primed = diagnostic_read(q_filenames(i, :)) ;

		q_primed(:, N_azim+1, :) = q_primed(:, 1, :) ;		% to join up data across final azimuthal gridpoint
		contourf(x_mat(:,:), y_mat(:,:), q_primed(:, :, layer), 20) ;
		shading flat
		colorbar_handle = colorbar ;
		set(get(colorbar_handle,'YLabel'),'String', ...
			['q '' (r, \theta, layer=', num2str(layer), ', t=', num2str(inds(i)*delta_t), 's)   (s^-^1)']) ;
		axis([-1.1*b, 1.1*b, -1.1*b, 1.1*b])
		title([wd, '   (F, d) = (', num2str(F), ', ', num2str(d), ')']) ;
		xlabel('x (m)') ;
		ylabel('y (m)') ;
		axis equal
		grid on
		pause(0)
		if (iteration ~= 0)
			M(i) = getframe;
		end

	end

	if (iteration ~= 0)
		movie(M, iteration, fps)
	end

elseif option == 3 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

	% contour plot of perturbation interface height (neglecting interfacial tension and centripetal effects)

	x_mat(:, N_azim+1) = x_mat(:, 1) ;				% } to join up data across final azimuthal gridpoint
	y_mat(:, N_azim+1) = y_mat(:, 1) ;				% }

	for i = 1 : length(inds)

		psi_primed = diagnostic_read(psi_filenames(i, :)) ;
		eta_primed = f/g_primed*(psi_primed(:, :, 2) - psi_primed(:, :, 1)) ;
		eta_primed = eta_primed - mean(mean(eta_primed .* r_mat .* area_factor)) / mean(mean(r_mat .* area_factor)) ;
												% subtract <eta'> to ensure volume conservation

		eta_primed(:, N_azim+1) = eta_primed(:, 1) ;					% to join up data across final azimuthal gridpoint
		contourf(x_mat(:,:), y_mat(:,:), 1000*eta_primed, 20) ;
		shading flat
		colorbar_handle = colorbar ;
		set(get(colorbar_handle,'YLabel'),'String', ...
			['\eta '' (r, \theta, t=', num2str(inds(i)*delta_t), 's)   (mm)']) ;
		title([wd, '   (F, d) = (', num2str(F), ', ', num2str(d), ')']) ;
		axis([-1.1*b, 1.1*b, -1.1*b, 1.1*b])
		xlabel('x (m)') ;
		ylabel('y (m)') ;
		axis equal
		grid on
		pause(0)  
		if (iteration ~= 0)
			M(i) = getframe;
		end

	end

	if (iteration ~= 0)
		movie(M, iteration, fps)
	end

elseif option == 4 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
	% vector plot of perturbation velocity field

	for i = 1 : length(inds)
                
		psi_primed = diagnostic_read(psi_filenames(i, :)) ;
       
		[dpsi_dtheta, dpsi_dr] = gradient_imp(psi_primed(:, :, layer), delta_theta, delta_r) ;

		u_r = -dpsi_dtheta./r_mat ;
		u_theta = dpsi_dr ;
		u_x = u_r.*x_mat./sqrt(x_mat.*x_mat+y_mat.*y_mat) - u_theta.*y_mat./sqrt(x_mat.*x_mat+y_mat.*y_mat) ;
		u_y = u_r.*y_mat./sqrt(x_mat.*x_mat+y_mat.*y_mat) + u_theta.*x_mat./sqrt(x_mat.*x_mat+y_mat.*y_mat) ;

		quiver(x_mat(:,:), y_mat(:,:), u_x(:,:), u_y(:,:), 'k')
		axis([-1.1*b, 1.1*b, -1.1*b, 1.1*b])
		title([wd, '   (F, d) = (', num2str(F), ', ', num2str(d), ')']) ;
		xlabel('x (m)') ;
		ylabel('y (m)') ;
		axis equal
		text(-1.2*b, -b, ['u '' (r, \theta, layer=', num2str(layer), ', t=', num2str(inds(i)*delta_t), 's)'], 'Rotation', 90) ;
		text(-1.2*b, -b, 'u', 'FontWeight', 'Bold', 'Rotation', 90) ;
		grid on
		pause(0)

		if (iteration ~= 0)
			M(i) = getframe;
		end

	end

	fprintf(1, ['\n     maximum radial perturbation velocity is: ' num2str(1000*max(max(abs(u_r)))) ' mm/s']) ;
	fprintf(1, ['\n     maximum zonal perturbation velocity is: ' num2str(1000*max(max(abs(u_theta)))) ' mm/s']) ;

	fprintf(1, ['\n     rms radial perturbation velocity is: ' num2str(1000*sqrt(mean(mean(u_r.*u_r)))) ' mm/s']) ;
	fprintf(1, ['\n     rms zonal perturbation velocity is: ' num2str(1000*sqrt(mean(mean(u_theta.*u_theta)))) ' mm/s']) ;

	if (iteration ~= 0)
		movie(M, iteration, fps)
	end

elseif option == 5 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
	% contour plot of full streamfunction

	x_mat(:, N_azim+1) = x_mat(:, 1) ;				% } to join up data across final azimuthal gridpoint
	y_mat(:, N_azim+1) = y_mat(:, 1) ;				% }

	psi_bar = zeros(N_rad, N_azim+1, N_layer) ;
	for i = 1 : N_rad
		for k = 1 : N_layer
			psi_bar(i, :, k) = 0.5*delta_omega(k)*r(i)*r(i) ;
		end
	end

	for i = 1 : length(inds)

		psi_primed = diagnostic_read(psi_filenames(i, :)) ;

		psi_primed(:, N_azim+1, :) = psi_primed(:, 1, :) ;	% to join up data across final azimuthal gridpoint
		contourf(x_mat(:,:), y_mat(:,:), psi_bar(:, :, layer) + psi_primed(:, :, layer), 20) ;
		shading flat
		colorbar_handle = colorbar ;
		set(get(colorbar_handle,'YLabel'),'String', ...
			['\psi (r, \theta, layer=', num2str(layer), ', t=', num2str(inds(i)*delta_t), 's)   (m^2 s^-^1)']) ;
		title([wd, '   (F, d) = (', num2str(F), ', ', num2str(d), ')']) ;
		axis([-1.1*b, 1.1*b, -1.1*b, 1.1*b])
		xlabel('x (m)') ;
		ylabel('y (m)') ;
		axis equal
		grid on
		pause(0)
		if (iteration ~= 0)
			M(i) = getframe;
		end

	end

	if (iteration ~= 0)
		movie(M, iteration, fps)
	end

elseif option == 6 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
	% contour plot of full potential vorticity

	x_mat(:, N_azim+1) = x_mat(:, 1) ;				% } to join up data across final azimuthal gridpoint
	y_mat(:, N_azim+1) = y_mat(:, 1) ;				% }

	q_bar = zeros(N_rad, N_azim+1, N_layer) ;
	for i = 1 : N_rad
		for k = 1 : N_layer
			q_bar(i, :, k) = f + (-1)^(k+1)*f*f*(omega/g - lid_delta_omega/g_primed)*r(i)*r(i)/(4*H) ...
					+ 2*delta_omega(k) + (-1)^(k)*f*f*delta_m*delta_m*lid_delta_omega/(g_primed*H) ;
		end
		q_bar(i, :, 1) = q_bar(i, :, 1) - f*s_top*(r(i)-a)/H ;
		q_bar(i, :, 2) = q_bar(i, :, 2) + f*s_bot*(r(i)-a)/H ;
	end

	for i = 1 : length(inds)

		q_primed = diagnostic_read(q_filenames(i, :)) ;

		q_primed(:, N_azim+1, :) = q_primed(:, 1, :) ;		% to join up data across final azimuthal gridpoint
		contourf(x_mat(:,:), y_mat(:,:), q_bar(:, :, layer) + q_primed(:, :, layer), 20) ;
		shading flat
		colorbar_handle = colorbar ;
		set(get(colorbar_handle,'YLabel'),'String', ...
			['q (r, \theta, layer=', num2str(layer), ', t=', num2str(inds(i)*delta_t), 's)   (s^-^1)']) ;
		title([wd, '   (F, d) = (', num2str(F), ', ', num2str(d), ')']) ;
		axis([-1.1*b, 1.1*b, -1.1*b, 1.1*b])
		xlabel('x (m)') ;
		ylabel('y (m)') ;
		axis equal
		grid on
		pause(0)
		if (iteration ~= 0)
			M(i) = getframe;
		end
           
	end

	if (iteration ~= 0)
		movie(M, iteration, fps)
	end

elseif option == 7 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
	% contour plot of full interface height (neglecting interfacial tension but including centripetal effect)

        x_mat(:, N_azim+1) = x_mat(:, 1) ;                              % } to join up data across final azimuthal gridpoint
        y_mat(:, N_azim+1) = y_mat(:, 1) ;                              % }

	eta_bar = zeros(N_rad, N_azim) ;
	for i = 1 : N_rad
		eta_bar(i, :) = omega*omega*r(i)*r(i)*(1.0 - (lid_delta_omega/omega)/(delta_rho/rho_bar))/(2.0*g) ;
	end

	for i = 1 : length(inds)

		psi_primed = diagnostic_read(psi_filenames(i, :)) ;
		eta = eta_bar + f/g_primed*(psi_primed(:, :, 2) - psi_primed(:, :, 1)) ;
		eta = eta - mean(mean(eta .* r_mat .* area_factor)) / mean(mean(r_mat .* area_factor)) ;	% subtract <eta> to ensure volume conservation

		eta(:, N_azim+1) = eta(:, 1) ;				% to join up data across final azimuthal gridpoint
		contourf(x_mat(:,:), y_mat(:,:), 1000*(H+eta), 20) ;
		shading flat
		colorbar_handle = colorbar ;
		set(get(colorbar_handle,'YLabel'),'String', ...
			['h (r, \theta, t=', num2str(inds(i)*delta_t), 's)   (mm)']) ;
		title([wd, '   (F, d) = (', num2str(F), ', ', num2str(d), ')']) ;
		axis([-1.1*b, 1.1*b, -1.1*b, 1.1*b])
		xlabel('x (m)') ;
		ylabel('y (m)') ;
		axis equal
		grid on
		pause(0)
		if (iteration ~= 0)
			M(i) = getframe;
		end

	end

	if (iteration ~= 0)
		movie(M, iteration, fps)
	end

elseif option == 8 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
	% vector plot of full velocity field

	psi_bar = zeros(N_rad, N_azim, N_layer) ;
	for i = 1 : N_rad
		for k = 1 : N_layer
			psi_bar(i, :, k) = 0.5*delta_omega(k)*r(i)*r(i) ;
		end
	end

	for i = 1 : length(inds)
                   
		psi_primed = diagnostic_read(psi_filenames(i, :)) ;

		[dpsi_dtheta, dpsi_dr] = gradient_imp(psi_primed(:, :, layer) + psi_bar(:, :, layer), delta_theta, delta_r) ;
		u_r = -dpsi_dtheta./r_mat ;
		u_theta = dpsi_dr ;
		u_x = u_r.*x_mat./sqrt(x_mat.*x_mat+y_mat.*y_mat) - u_theta.*y_mat./sqrt(x_mat.*x_mat+y_mat.*y_mat) ;
		u_y = u_r.*y_mat./sqrt(x_mat.*x_mat+y_mat.*y_mat) + u_theta.*x_mat./sqrt(x_mat.*x_mat+y_mat.*y_mat) ;

		quiver(x_mat(:,:), y_mat(:,:), u_x(:,:), u_y(:,:), 'k')
		axis([-1.1*b, 1.1*b, -1.1*b, 1.1*b])
		text(-1.2*b, -b, ['u (r, \theta, layer=', num2str(layer), ', t=', num2str(inds(i)*delta_t), 's)'], 'Rotation', 90) ;
		text(-1.2*b, -b, 'u', 'FontWeight', 'Bold', 'Rotation', 90) ;
		title([wd, '   (F, d) = (', num2str(F), ', ', num2str(d), ')']) ;
		xlabel('x (m)') ;
		ylabel('y (m)') ;
		axis equal
		grid on
		pause(0)
                
		if (iteration ~= 0)
			M(i) = getframe;
		end
                
	end

	fprintf(1, ['\n     maximum radial velocity is: ' num2str(1000*max(max(abs(u_r)))) ' mm/s']) ;
	fprintf(1, ['\n     maximum zonal velocity is: ' num2str(1000*max(max(abs(u_theta)))) ' mm/s']) ;

	fprintf(1, ['\n     rms radial velocity is: ' num2str(1000*sqrt(mean(mean(u_r.*u_r)))) ' mm/s']) ;
	fprintf(1, ['\n     rms zonal velocity is: ' num2str(1000*sqrt(mean(mean(u_theta.*u_theta)))) ' mm/s']) ;
                
	if (iteration ~= 0)
		movie(M, iteration, fps)
	end

elseif option == 9 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

	% time-series of mid-radius interface height wave amplitude (neglecting tension)

	amp = [] ;
	for i = 1 : length(inds)
		psi_primed = diagnostic_read(psi_filenames(i,:)) ;
		eta_primed = f / g_primed * ( psi_primed(floor(N_rad/2),:,2) - psi_primed(floor(N_rad/2),:,1) ) ;
                eta_primed_filteredx3 = filter(ones(1, windowSize)/windowSize, 1, [eta_primed,eta_primed,eta_primed]) ;
                eta_primed = eta_primed_filteredx3(:, N_azim+1+floor(windowSize/2) : 2*N_azim+floor(windowSize/2)) ; 
		amp = [amp, (max(eta_primed) - min(eta_primed)) / 2.0] ;
	end

	t = inds*delta_t ;

	plot(t, 1000*amp, 'LineWidth', 2) ;
	ylabel(['amplitude of \eta '' (r=mid-radius, \theta, t=fixed)   (mm)']) ;
	xlabel('t (s)') ;
	title([wd, '   (F, d) = (', num2str(F), ', ', num2str(d), ')']) ;
	grid on ;

	fprintf(1, ['\n     minimum amplitude is: ' num2str(1000*min(amp)) ' mm']) ;
	fprintf(1, ['\n     mean amplitude (+/- standard deviation) is: ' num2str(1000*mean(amp)) ' +/- ' num2str(1000*std(amp)) ' mm']) ;
	fprintf(1, ['\n     maximum amplitude is: ' num2str(1000*max(amp)) ' mm']) ;
	fprintf(1, ['\n     amplitude at end of series is: ' num2str(1000*amp(length(inds))) ' mm']) ;

	FFT = abs(fft(amp)) ;
	FFT(1) = 0.0 ;
	FFT = FFT(1 : floor(length(inds)/2)) ;
	[useless, useful] = max(FFT) ;   
	est_period = (t(length(inds)) - t(1))/(useful-1.0) ;
	error = ((t(length(inds)) - t(1))/(useful-1.5) - (t(length(inds)) - t(1))/(useful-0.5))/2.0 ;
	fprintf(1, ['\n     period of signal in figure is: (' num2str(est_period) ' +/- ' num2str(error) ') s']) ;

elseif option == 10 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

	% multiple cartesian diagnostics

	for i = 1 : length(inds)

		q_primed = diagnostic_read(q_filenames(i, :)) ;

		subplot(3, 2, 1)
		set(gcf, 'Position', [200 100 800 580]) ;
		pcolor(theta_mat(:,:), r_mat(:,:), q_primed(:, :, layer)) ;
		title(['q '' (r, \theta, layer=' num2str(layer) ', t=', num2str(inds(i)*delta_t), 's)   (s^-^1)']) ;
		xlabel('\theta (rad)')
		ylabel('r (m)') ;
		colorbar
		% shading interp
	
		subplot(3, 2, 3)
		plot(q_primed(:, 1, layer), r(:), 'o') ;
		grid on
		xlabel('cross-section through \theta=0')
		ylabel('r (m)') ;
		set(gca, 'YLim', [a b]) ;
	
		subplot(3, 2, 5)
		plot(theta(:), q_primed(floor(N_rad/2), :, layer), 'o') ;
		grid on
		ylabel('cross-section at mid radius')
		xlabel('\theta (rad)') ;
		set(gca, 'XLim', [0 2*pi]) ;
	
		psi_primed = diagnostic_read(psi_filenames(i,:)) ;

		subplot(3, 2, 2)
		pcolor(theta_mat(:,:), r_mat(:,:), psi_primed(:, :, layer)) ;
		title(['\psi '' (r, \theta, layer=' num2str(layer) ', t=', num2str(inds(i)*delta_t), 's)   (m^2 s^-^1)']) ;
		xlabel('\theta (rad)')
		ylabel('r (m)') ;
		colorbar
		% shading interp
	
		subplot(3, 2, 4)
		plot(psi_primed(:, 1, layer), r(:), 'o') ;
		grid on
		xlabel('cross-section through \theta=0')
		ylabel('r (m)') ;
		set(gca, 'YLim', [a b]) ;
	
		subplot(3, 2, 6)
		plot(theta(:), psi_primed(floor(N_rad/2), :, layer), 'o') ;
		grid on
		ylabel('cross-section at mid radius')
		xlabel('\theta (rad)') ;
		set(gca, 'XLim', [0 2*pi]) ;
	
		pause(0)
	
	end

elseif option == 11 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

	% plot of gridpoint positions

	plot(x_mat(:,:), y_mat(:,:), '.k') ;
	axis([-1.1*b, 1.1*b, -1.1*b, 1.1*b]) ;
	axis equal
	grid on
	xlabel('x (m)');
	ylabel('y (m)');

elseif option == 12 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

	% time-series of mid-radius interface height (to within an additive constant, and neglecting tension)

	eta_primed_series = [] ;
	for i = 1 : length(inds)
		psi_primed = diagnostic_read(psi_filenames(i,:)) ;
		eta_primed = f/g_primed*(psi_primed(:,:,2)-psi_primed(:,:,1)) ;
		eta_primed = eta_primed - mean(mean(eta_primed .* r_mat .* area_factor)) / mean(mean(r_mat .* area_factor)) ; 
													% subtract <eta'> to ensure volume conservation
		eta_primed_series = [eta_primed_series, eta_primed(floor(N_rad/2),1)] ;
	end

	t = inds*delta_t ;

	plot(t, 1000*eta_primed_series) ;
	ylabel(['\eta '' (r=mid-radius, \theta=0, t)   (mm)']) ;
	xlabel('t (s)') ;
	title([wd, '   (F, d) = (', num2str(F), ', ', num2str(d), ')']) ;
	grid on ;

	FFT = abs(fft(eta_primed_series)) ;
	FFT(1) = 0.0 ;
	FFT = FFT(1 : floor(length(inds)/2)) ;
	[useless, useful] = max(FFT) ;
	est_period = (t(length(inds)) - t(1))/(useful-1.0) ;
	error = ((t(length(inds)) - t(1))/(useful-1.5) - (t(length(inds)) - t(1))/(useful-0.5))/2.0 ;
	fprintf(1, ['\n     period of signal in figure is: (' num2str(est_period) ' +/- ' num2str(error) ') s']) ;

elseif option == 13 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

	% option 1 over-plotted with option 4

	x_mat_contour = x_mat ;						% } the two plots require slightly different grids
	y_mat_contour = y_mat ;						% }

	x_mat_contour(:, N_azim+1) = x_mat_contour(:, 1) ;			% } to join up data across final azimuthal gridpoint
	y_mat_contour(:, N_azim+1) = y_mat_contour(:, 1) ;			% }
        
	for i = 1 : length(inds)
                
		psi_primed = diagnostic_read(psi_filenames(i, :)) ;

		[dpsi_dtheta, dpsi_dr] = gradient_imp(psi_primed(:, :, layer), delta_theta, delta_r) ;
		u_r = -dpsi_dtheta./r_mat ;
		u_theta = dpsi_dr ;
		u_x = u_r.*x_mat./sqrt(x_mat.*x_mat+y_mat.*y_mat) - u_theta.*y_mat./sqrt(x_mat.*x_mat+y_mat.*y_mat) ;
		u_y = u_r.*y_mat./sqrt(x_mat.*x_mat+y_mat.*y_mat) + u_theta.*x_mat./sqrt(x_mat.*x_mat+y_mat.*y_mat) ;

		psi_primed(:, N_azim+1, :) = psi_primed(:, 1, :) ;      % to join up data across final azimuthal gridpoint
		contourf(x_mat_contour(:,:), y_mat_contour(:,:), psi_primed(:, :, layer), 20) ;
		hold on
		quiver(x_mat(:,:), y_mat(:,:), u_x(:,:), u_y(:,:), 'k')
		shading flat
		colorbar_handle = colorbar ;
		set(get(colorbar_handle,'YLabel'),'String', ...
			['\psi '' (r, \theta, layer=', num2str(layer), ', t=', num2str(inds(i)*delta_t), 's)   (m^2 s^-^1)']) ;
		title([wd, '   (F, d) = (', num2str(F), ', ', num2str(d), ')']) ;
		axis([-1.1*b, 1.1*b, -1.1*b, 1.1*b])
		xlabel('x (m)') ;
		ylabel('y (m)') ;
		axis equal
		grid on
		pause(0)
		if (iteration ~= 0)
			M(i) = getframe;
		end

	end
        
	if (iteration ~= 0)
		movie(M, iteration, fps)
	end

elseif option == 14 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                
	% option 5 over-plotted with option 8
                
	x_mat_contour = x_mat ;						% } the two plots require slightly different grids
	y_mat_contour = y_mat ;						% }

	x_mat_contour(:, N_azim+1) = x_mat_contour(:, 1) ;		% } to join up data across final azimuthal gridpoint
	y_mat_contour(:, N_azim+1) = y_mat_contour(:, 1) ;		% }
                
	psi_bar = zeros(N_rad, N_azim, N_layer) ;
	for i = 1 : N_rad
		for k = 1 : N_layer
			psi_bar(i, :, k) = 0.5*delta_omega(k)*r(i)*r(i) ;
		end
	end

	psi_bar_contour = psi_bar ;					% streamfunctions on the two different grids
	psi_bar_contour(:, N_azim+1, :) = psi_bar_contour(:, 1, :) ;

	for i = 1 : length(inds)

		psi_primed = diagnostic_read(psi_filenames(i, :)) ;
                   
		[dpsi_dtheta, dpsi_dr] = gradient_imp(psi_primed(:, :, layer) + psi_bar(:, :, layer), delta_theta, delta_r) ;
		u_r = -dpsi_dtheta./r_mat ;
		u_theta = dpsi_dr ;
		u_x = u_r.*x_mat./sqrt(x_mat.*x_mat+y_mat.*y_mat) - u_theta.*y_mat./sqrt(x_mat.*x_mat+y_mat.*y_mat) ;
		u_y = u_r.*y_mat./sqrt(x_mat.*x_mat+y_mat.*y_mat) + u_theta.*x_mat./sqrt(x_mat.*x_mat+y_mat.*y_mat) ;

		psi_primed(:, N_azim+1, :) = psi_primed(:, 1, :) ;      % to join up data across final azimuthal gridpoint

		contourf(x_mat_contour(:,:), y_mat_contour(:,:), psi_primed(:, :, layer) + psi_bar_contour(:, :, layer), 20) ;
		hold on
		quiver(x_mat(:,:), y_mat(:,:), u_x(:,:), u_y(:,:), 'k')
		shading flat
		colorbar_handle = colorbar ;
		set(get(colorbar_handle,'YLabel'),'String', ...
			['\psi (r, \theta, layer=', num2str(layer), ', t=', num2str(inds(i)*delta_t), 's)   (m^2 s^-^1)']) ;
		title([wd, '   (F, d) = (', num2str(F), ', ', num2str(d), ')']) ;
		axis([-1.1*b, 1.1*b, -1.1*b, 1.1*b])
		xlabel('x (m)') ;
		ylabel('y (m)') ;
		axis equal
		grid on
		pause(0)
		if (iteration ~= 0)
			M(i) = getframe;
		end

	end
           
	if (iteration ~= 0)
		movie(M, iteration, fps)
	end

elseif option == 15 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

	% time-series of radially-averaged zonal perturbation velocity
                
	razv = [] ;
	for i = 1 : length(inds)
		psi_primed = diagnostic_read(psi_filenames(i,:)) ;
		razv = [razv, (psi_primed(N_rad, 1, :) - psi_primed(1, 1, :))/(b - a)] ;
	end
                
	t = inds*delta_t ;
	plot(t, 1000*squeeze(razv)) ;
	legend('layer 1', 'layer 2');
	ylabel('radially-averaged u_{\theta}'' (r, \theta=0, layer, t)   (mm s^-^1)') ;
	xlabel('t (s)') ;
	title([wd, '   (F, d) = (', num2str(F), ', ', num2str(d), ')']) ;
	grid on ;

	fprintf(1, ['\n     minimum velocities are: ' num2str(1000*min(razv(1, :, :))) ' mm/s']) ;
	fprintf(1, ['\n     mean velocities are: ' num2str(1000*mean(razv(1, :, :))) ' mm/s']) ;
	fprintf(1, ['\n     maximum velocities are: ' num2str(1000*max(razv(1, :, :))) ' mm/s']) ;
	fprintf(1, ['\n     velocities at end of series are: ' num2str(1000*razv(1, length(inds), :)) ' mm/s']) ;

elseif option == 16 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

	% mid-radius perturbation potential vorticity Hovmueller diagram

	q_primed = [] ;
	for i = 1 : length(inds)
		q_temp = diagnostic_read(q_filenames(i, :)) ;
		q_primed = [q_primed ; q_temp(floor(N_rad/2), :, layer)] ;
	end

	q_primed(:,N_azim+1) = q_primed(:,1) ;
	theta(N_azim+1) = 2*pi ;
	contourf(inds*delta_t, theta*180/pi, q_primed(:, :)', 20) ;
	set(gca, 'Box', 'on') ;
	shading flat
	colorbar_handle = colorbar ;
	set(get(colorbar_handle,'YLabel'),'String', ...
		['q '' (r=mid-radius, \theta, layer=', num2str(layer), ', t)   (s^-^1)']) ;
	title([wd, '   (F, d) = (', num2str(F), ', ', num2str(d), ')']) ;
	xlabel('t (s)') ;
	ylabel('\theta (degrees)') ;
	set(gca, 'YTick', 0:60:360) ;
	set(gcf, 'PaperPosition', [0.25 2.5 16 6]) ;	% use a font size of 20

elseif option == 17 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

	% mid-radius perturbation interface height Fourier spectrum

	for i = 1 : length(inds)

		psi_primed = diagnostic_read(psi_filenames(i, :)) ;

		FFT = abs(fft( psi_primed(floor(N_rad/2), :, 2) - psi_primed(floor(N_rad/2), :, 1) )) ;
		FFT(1) = nan ;				% wavenumber 0 component is irrelevant
		FFT = FFT(1 : floor(N_azim/2 + 1)) ;
		FFT = FFT / FFT(2) ;			% normalise by wavenumber 1 component

		semilogy([0:floor(N_azim/2)], FFT, 'x') ;        
		title([wd, '   (F, d) = (', num2str(F), ', ', num2str(d), ')']) ;
		xlabel('wavenumber') ;
		ylabel(['FFT of \eta '' (r=mid-radius, \theta, t=',num2str(inds(i)*delta_t),'s)']);
		grid on
		pause(0)
		if (iteration ~= 0)
			M(i) = getframe;
		end
        
	end
        
	if (iteration ~= 0)
		movie(M, iteration, fps)
	end

elseif option == 18 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

	% contour plot of Lighthill radiation term

	psi_bar = zeros(N_rad, N_azim, N_layer) ;
	for i = 1 : N_rad
		for k = 1 : N_layer
			psi_bar(i, :, k) = 0.5*delta_omega(k)*r(i)*r(i) ;
		end
	end

        eta_bar = zeros(N_rad, N_azim) ;
        for i = 1 : N_rad
                eta_bar(i, :) = omega*omega*r(i)*r(i)*(1.0 - (lid_delta_omega/omega)/(delta_rho/rho_bar))/(2.0*g) ;
        end     

	x_mat(:, N_azim+1) = x_mat(:, 1) ;                              % } to join up data across final azimuthal gridpoint
	y_mat(:, N_azim+1) = y_mat(:, 1) ;                              % }

	for i = 1 : length(inds)

		% PREVIOUS TIME
		% read in data
			if i == 1
				tmp_PSI_ = ['PSI_.0000000000'] ;
				tmp_PSI_(16-length(num2str(start-increment)) : 15) = num2str(start-increment) ;
				psi_primed = diagnostic_read(tmp_PSI_) ;
			else    
				psi_primed = diagnostic_read(psi_filenames(i-1, :)) ;
			end     
		psi_primed_filteredx3 = filter(ones(1, windowSize)/windowSize, 1, [psi_primed, psi_primed, psi_primed], [], 2) ;
		psi_primed = psi_primed_filteredx3(:, N_azim+1+floor(windowSize/2) : 2*N_azim+floor(windowSize/2), :) ;
		% calculate u_r and u_theta in layer 2
			[dpsi_dtheta, dpsi_dr] = gradient_imp(psi_primed(:, :, 2) + psi_bar(:, :, 2), delta_theta, delta_r) ;
			u_r = -dpsi_dtheta./r_mat ;
			u_theta = dpsi_dr ;
		% calculate h
			eta = eta_bar + f/g_primed*(psi_primed(:, :, 2) - psi_primed(:, :, 1)) ;
			eta = eta - mean(mean(eta .* r_mat .* area_factor)) / mean(mean(r_mat .* area_factor)) ;     % subtract <eta> to ensure volume conservation
			h = H + eta ;
		% calculate some gradients which are needed to calculate F
			[useless, d_dr_of_hu_r] = gradient_imp(h.*u_r, delta_theta, delta_r) ;
			[d_dtheta_of_hu_theta, useless] = gradient_imp(h.*u_theta, delta_theta, delta_r) ;
			[d_dtheta_of_u_r, d_dr_of_u_r] = gradient_imp(u_r, delta_theta, delta_r) ;
			[d_dtheta_of_u_theta, d_dr_of_u_theta] = gradient_imp(u_theta, delta_theta, delta_r) ;
		% calculate components of F
			F_r = u_r .* ( h .* u_r ./ r_mat + d_dr_of_hu_r + d_dtheta_of_hu_theta ./ r_mat ) ...
				+ h .* ( u_r .* d_dr_of_u_r + u_theta ./ r_mat .* d_dtheta_of_u_r - u_theta .* u_theta ./ r_mat ) ;
			F_theta = u_theta .* ( h .* u_r ./ r_mat + d_dr_of_hu_r + d_dtheta_of_hu_theta ./ r_mat ) ...
				+ h .* ( u_r .* d_dr_of_u_theta + u_theta ./ r_mat .* d_dtheta_of_u_theta + u_theta .* u_r ./ r_mat ) ;
		% calculate some gradients of F
			[d_dtheta_of_F_r, d_dr_of_F_r] = gradient_imp(F_r, delta_theta, delta_r) ;
			[d_dtheta_of_F_theta, d_dr_of_F_theta] = gradient_imp(F_theta, delta_theta, delta_r) ;
                % calculate some gradients required for Laplacian(eta^2)
                        [d_dtheta_of_eta2, d_dr_of_eta2] = gradient_imp(eta.*eta, delta_theta, delta_r) ;
                        [d2_dr_dtheta_of_eta2, d2_dr2_of_eta2] = gradient_imp(d_dr_of_eta2, delta_theta, delta_r) ;
                        [d2_dtheta2_of_eta2, d2_dr_dtheta_of_eta2] = gradient_imp(d_dtheta_of_eta2, delta_theta, delta_r) ;
		% give special names to the variables we wish to retain at this timestep
			div_F_previous = F_r ./ r_mat + d_dr_of_F_r + d_dtheta_of_F_theta ./ r_mat ;
			k_dot_curl_F_previous = F_theta ./ r_mat + d_dr_of_F_theta - d_dtheta_of_F_r ./ r_mat ;
			del2_eta2_previous = d2_dr2_of_eta2 + d_dr_of_eta2./r_mat + d2_dtheta2_of_eta2./r_mat./r_mat ;

		% CURRENT TIME
		% read in data
			psi_primed = diagnostic_read(psi_filenames(i, :)) ;
		psi_primed_filteredx3 = filter(ones(1, windowSize)/windowSize, 1, [psi_primed, psi_primed, psi_primed], [], 2) ;
		psi_primed = psi_primed_filteredx3(:, N_azim+1+floor(windowSize/2) : 2*N_azim+floor(windowSize/2), :) ;
		% calculate u_r and u_theta in layer 2
			[dpsi_dtheta, dpsi_dr] = gradient_imp(psi_primed(:, :, 2) + psi_bar(:, :, 2), delta_theta, delta_r) ;
			u_r = -dpsi_dtheta./r_mat ;
			u_theta = dpsi_dr ;
		% calculate h
			eta = eta_bar + f/g_primed*(psi_primed(:, :, 2) - psi_primed(:, :, 1)) ;
			eta = eta - mean(mean(eta .* r_mat .* area_factor)) / mean(mean(r_mat .* area_factor)) ;      % subtract <eta> to ensure volume conservation
			h = H + eta ;
		% calculate some gradients which are needed to calculate F
			[useless, d_dr_of_hu_r] = gradient_imp(h.*u_r, delta_theta, delta_r) ;
			[d_dtheta_of_hu_theta, useless] = gradient_imp(h.*u_theta, delta_theta, delta_r) ;
			[d_dtheta_of_u_r, d_dr_of_u_r] = gradient_imp(u_r, delta_theta, delta_r) ;
			[d_dtheta_of_u_theta, d_dr_of_u_theta] = gradient_imp(u_theta, delta_theta, delta_r) ;
		% calculate components of F
			F_r = u_r .* ( h .* u_r ./ r_mat + d_dr_of_hu_r + d_dtheta_of_hu_theta ./ r_mat ) ...
				+ h .* ( u_r .* d_dr_of_u_r + u_theta ./ r_mat .* d_dtheta_of_u_r - u_theta .* u_theta ./ r_mat ) ;
			F_theta = u_theta .* ( h .* u_r ./ r_mat + d_dr_of_hu_r + d_dtheta_of_hu_theta ./ r_mat ) ...
				+ h .* ( u_r .* d_dr_of_u_theta + u_theta ./ r_mat .* d_dtheta_of_u_theta + u_theta .* u_r ./ r_mat ) ;
		% calculate some gradients of F
			[d_dtheta_of_F_r, d_dr_of_F_r] = gradient_imp(F_r, delta_theta, delta_r) ;
			[d_dtheta_of_F_theta, d_dr_of_F_theta] = gradient_imp(F_theta, delta_theta, delta_r) ;
                % calculate some gradients required for Laplacian(eta^2)
                        [d_dtheta_of_eta2, d_dr_of_eta2] = gradient_imp(eta.*eta, delta_theta, delta_r) ;
                        [d2_dr_dtheta_of_eta2, d2_dr2_of_eta2] = gradient_imp(d_dr_of_eta2, delta_theta, delta_r) ;
                        [d2_dtheta2_of_eta2, d2_dr_dtheta_of_eta2] = gradient_imp(d_dtheta_of_eta2, delta_theta, delta_r) ;
		% give special names to the variables we wish to retain at this timestep
			div_F_current = F_r ./ r_mat + d_dr_of_F_r + d_dtheta_of_F_theta ./ r_mat ;
			k_dot_curl_F_current = F_theta ./ r_mat + d_dr_of_F_theta - d_dtheta_of_F_r ./ r_mat ;
			del2_eta2_current = d2_dr2_of_eta2 + d_dr_of_eta2./r_mat + d2_dtheta2_of_eta2./r_mat./r_mat ;

		% NEXT TIME
		% read in data
			if i == length(inds)
				tmp_PSI_ = ['PSI_.0000000000'] ;
				tmp_PSI_(16-length(num2str(finish+increment)) : 15) = num2str(finish+increment) ;
				psi_primed = diagnostic_read(tmp_PSI_) ;
			else    
				psi_primed = diagnostic_read(psi_filenames(i+1, :)) ;
			end
		psi_primed_filteredx3 = filter(ones(1, windowSize)/windowSize, 1, [psi_primed, psi_primed, psi_primed], [], 2) ;
		psi_primed = psi_primed_filteredx3(:, N_azim+1+floor(windowSize/2) : 2*N_azim+floor(windowSize/2), :) ;
		% calculate u_r and u_theta in layer 2
			[dpsi_dtheta, dpsi_dr] = gradient_imp(psi_primed(:, :, 2) + psi_bar(:, :, 2), delta_theta, delta_r) ;
			u_r = -dpsi_dtheta./r_mat ;
			u_theta = dpsi_dr ;
		% calculate h
			eta = eta_bar + f/g_primed*(psi_primed(:, :, 2) - psi_primed(:, :, 1)) ;
			eta = eta - mean(mean(eta .* r_mat .* area_factor)) / mean(mean(r_mat .* area_factor)) ;      % subtract <eta> to ensure volume conservation
			h = H + eta ;
		% calculate some gradients which are needed to calculate F
			[useless, d_dr_of_hu_r] = gradient_imp(h.*u_r, delta_theta, delta_r) ;
			[d_dtheta_of_hu_theta, useless] = gradient_imp(h.*u_theta, delta_theta, delta_r) ;
			[d_dtheta_of_u_r, d_dr_of_u_r] = gradient_imp(u_r, delta_theta, delta_r) ;
			[d_dtheta_of_u_theta, d_dr_of_u_theta] = gradient_imp(u_theta, delta_theta, delta_r) ;
		% calculate components of F
			F_r = u_r .* ( h .* u_r ./ r_mat + d_dr_of_hu_r + d_dtheta_of_hu_theta ./ r_mat ) ...
				+ h .* ( u_r .* d_dr_of_u_r + u_theta ./ r_mat .* d_dtheta_of_u_r - u_theta .* u_theta ./ r_mat ) ;
			F_theta = u_theta .* ( h .* u_r ./ r_mat + d_dr_of_hu_r + d_dtheta_of_hu_theta ./ r_mat ) ...
				+ h .* ( u_r .* d_dr_of_u_theta + u_theta ./ r_mat .* d_dtheta_of_u_theta + u_theta .* u_r ./ r_mat ) ;
		% calculate some gradients of F
			[d_dtheta_of_F_r, d_dr_of_F_r] = gradient_imp(F_r, delta_theta, delta_r) ;
			[d_dtheta_of_F_theta, d_dr_of_F_theta] = gradient_imp(F_theta, delta_theta, delta_r) ;
		% calculate some gradients required for Laplacian(eta^2)
			[d_dtheta_of_eta2, d_dr_of_eta2] = gradient_imp(eta.*eta, delta_theta, delta_r) ;
			[d2_dr_dtheta_of_eta2, d2_dr2_of_eta2] = gradient_imp(d_dr_of_eta2, delta_theta, delta_r) ;
			[d2_dtheta2_of_eta2, d2_dr_dtheta_of_eta2] = gradient_imp(d_dtheta_of_eta2, delta_theta, delta_r) ;
		% give special names to the variables we wish to retain at this timestep
			div_F_next = F_r ./ r_mat + d_dr_of_F_r + d_dtheta_of_F_theta ./ r_mat ;
			k_dot_curl_F_next = F_theta ./ r_mat + d_dr_of_F_theta - d_dtheta_of_F_r ./ r_mat ;
			del2_eta2_next = d2_dr2_of_eta2 + d_dr_of_eta2./r_mat + d2_dtheta2_of_eta2./r_mat./r_mat ;

		% calculate Lighthill radiation term
		LRT = ( div_F_next - div_F_previous ) / (2*delta_t*increment) ...
			+ f * k_dot_curl_F_current ...
			- g_primed/4 * (del2_eta2_next - del2_eta2_previous) / (2*delta_t*increment) ;

		% plot Lighthill radiation term
		LRT(:, N_azim+1) = LRT(:, 1) ;                          % to join up data across final azimuthal gridpoint
		contourf(x_mat(:,:), y_mat(:,:), abs(LRT(:, :)), 20) ;
		shading flat
		colorbar_handle = colorbar ;
		set(get(colorbar_handle,'YLabel'),'String', ...
			['|LRT| (r, \theta, t=', num2str(inds(i)*delta_t), 's)   (m s^-^3)']) ;
		title([wd, '   (F, d) = (', num2str(F), ', ', num2str(d), ')']) ;
		axis([-1.1*b, 1.1*b, -1.1*b, 1.1*b])
		xlabel('x (m)') ;
		ylabel('y (m)') ;
		axis equal
		grid on 
		pause(0)
                        
		if (iteration ~= 0)
			M(i) = getframe;
		end

	end
                
	if (iteration ~= 0)
		movie(M, iteration, fps)
	end

elseif option == 19 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

	% contour plot of Brown's CAT indicator (phi)

	psi_bar = zeros(N_rad, N_azim, N_layer) ;
	for i = 1 : N_rad
		for k = 1 : N_layer
			psi_bar(i, :, k) = 0.5*delta_omega(k)*r(i)*r(i) ;
		end
	end
                
	x_mat(:, N_azim+1) = x_mat(:, 1) ;				% } to join up data across final azimuthal gridpoint
	y_mat(:, N_azim+1) = y_mat(:, 1) ;				% }

	interfacial_Ekman_layer_depth = 2*sqrt(mean(new)/omega) ;

	for i = 1 : length(inds)

		% get Ri at time i-1
		if i == 1
			tmp_PSI_ = ['PSI_.0000000000'] ;
			tmp_PSI_(16-length(num2str(start-increment)) : 15) = num2str(start-increment) ;
			psi_primed = diagnostic_read(tmp_PSI_) ;
		else
			psi_primed = diagnostic_read(psi_filenames(i-1, :)) ;
		end
		[dpsi_dtheta_1, dpsi_dr_1] = gradient_imp(psi_primed(:, :, 1) + psi_bar(:, :, 1), delta_theta, delta_r) ;
		[dpsi_dtheta_2, dpsi_dr_2] = gradient_imp(psi_primed(:, :, 2) + psi_bar(:, :, 2), delta_theta, delta_r) ;
		u_r = -dpsi_dtheta_1./r_mat + dpsi_dtheta_2./r_mat;     % radial component of shear (top layer - bottom layer)
		u_theta = dpsi_dr_1 - dpsi_dr_2 ;                       % azimuthal component of shear (top layer - bottom layer)
		Ri_minus = g_primed * interfacial_Ekman_layer_depth ./ ( u_r .* u_r + u_theta .* u_theta ) ;
		Ri_minusx3 = filter(ones(1, windowSize)/windowSize, 1, [Ri_minus, Ri_minus, Ri_minus], [], 2) ;
		Ri_minus = Ri_minusx3(:, N_azim+1+floor(windowSize/2) : 2*N_azim+floor(windowSize/2)) ;

		% get Ri at time i
		psi_primed = diagnostic_read(psi_filenames(i, :)) ;
		[dpsi_dtheta_1, dpsi_dr_1] = gradient_imp(psi_primed(:, :, 1) + psi_bar(:, :, 1), delta_theta, delta_r) ;
		[dpsi_dtheta_2, dpsi_dr_2] = gradient_imp(psi_primed(:, :, 2) + psi_bar(:, :, 2), delta_theta, delta_r) ;
		u_r = -dpsi_dtheta_1./r_mat + dpsi_dtheta_2./r_mat;     % radial component of shear (top layer - bottom layer)
		u_theta = dpsi_dr_1 - dpsi_dr_2 ;                       % azimuthal component of shear (top layer - bottom layer)
		Ri_now = g_primed * interfacial_Ekman_layer_depth ./ ( u_r .* u_r + u_theta .* u_theta ) ;
		[dpsi_dtheta_now, dpsi_dr_now] = gradient_imp(psi_primed(:, :, layer) + psi_bar(:, :, layer), delta_theta, delta_r) ;
		u_r_now = -dpsi_dtheta_now./r_mat ;
		u_theta_now = dpsi_dr_now ;
		Ri_nowx3 = filter(ones(1, windowSize)/windowSize, 1, [Ri_now, Ri_now, Ri_now], [], 2) ;
		Ri_now = Ri_nowx3(:, N_azim+1+floor(windowSize/2) : 2*N_azim+floor(windowSize/2)) ;

		% get Ri at time i+1
		if i == length(inds)
			tmp_PSI_ = ['PSI_.0000000000'] ;
			tmp_PSI_(16-length(num2str(finish+increment)) : 15) = num2str(finish+increment) ;
			psi_primed = diagnostic_read(tmp_PSI_) ;
		else
			psi_primed = diagnostic_read(psi_filenames(i+1, :)) ;
		end
		[dpsi_dtheta_1, dpsi_dr_1] = gradient_imp(psi_primed(:, :, 1) + psi_bar(:, :, 1), delta_theta, delta_r) ;
		[dpsi_dtheta_2, dpsi_dr_2] = gradient_imp(psi_primed(:, :, 2) + psi_bar(:, :, 2), delta_theta, delta_r) ;
		u_r = -dpsi_dtheta_1./r_mat + dpsi_dtheta_2./r_mat;     % radial component of shear (top layer - bottom layer)
		u_theta = dpsi_dr_1 - dpsi_dr_2 ;                       % azimuthal component of shear (top layer - bottom layer)
		Ri_plus = g_primed * interfacial_Ekman_layer_depth ./ ( u_r .* u_r + u_theta .* u_theta ) ;
		Ri_plusx3 = filter(ones(1, windowSize)/windowSize, 1, [Ri_plus, Ri_plus, Ri_plus], [], 2) ;
		Ri_plus = Ri_plusx3(:, N_azim+1+floor(windowSize/2) : 2*N_azim+floor(windowSize/2)) ;

%		% calculate phi using D/Dt = d/dt + u.grad
%		phi = - (log(Ri_plus) - log(Ri_minus)) / (2*delta_t*increment) ;				% local rate of change
%		[dlnRi_dtheta_now, dlnRi_dr_now] = gradient_imp(log(Ri_now), delta_theta, delta_r) ;
%		phi = phi - ( - dpsi_dtheta_now .* dlnRi_dr_now + dpsi_dr_now .* dlnRi_dtheta_now ) ./ r_mat ;	% advection in a layer

		% calculate phi directly from definition using interpolation
		[theta_matrix, r_matrix, t_matrix] = meshgrid([theta, 2*pi+theta, 4*pi+theta], r, [-increment*delta_t, 0, increment*delta_t]) ;
		data_matrix(:,:,1) = [log(Ri_minus), log(Ri_minus), log(Ri_minus)] ;  
		data_matrix(:,:,2) = [log(Ri_now), log(Ri_now), log(Ri_now)] ;    
		data_matrix(:,:,3) = [log(Ri_plus), log(Ri_plus), log(Ri_plus)] ;
		% theta_matrix, r_matrix, t_matrix are all N_rad x 3*N_azim x 3
		% data_matrix(1:N_rad, 1:3*N_azim, 1:3) is log(Ri) at each gridpoint at each of three consecutive times, azimuthally tripled
		delta__t = delta_t ;	% the time which tends to zero
		for I=1:N_rad
			for J=1:N_azim
				phi(I,J) = - ( interp3(theta_matrix, r_matrix, t_matrix, data_matrix, ...
					        2*pi+theta(J)+u_theta_now(I,J)/r(I)*delta__t, r(I)+u_r_now(I,J)*delta__t, delta__t, '*linear') ... 
					       - data_matrix(I,J,2) ...
					     ) / delta__t ;
			end
		end

		% plot phi
		phi(:, N_azim+1) = phi(:, 1) ;
		contourf(x_mat(:,:), y_mat(:,:), phi(:, :), 20) ;
		shading flat
		colorbar_handle = colorbar ;
		set(get(colorbar_handle,'YLabel'),'String', ...
			['\phi (r, \theta, layer=', num2str(layer), ', t=', num2str(inds(i)*delta_t), 's)   (s^-^1)']) ;
		title([wd, '   (F, d) = (', num2str(F), ', ', num2str(d), ')']) ;
		axis([-1.1*b, 1.1*b, -1.1*b, 1.1*b])
		xlabel('x (m)') ;
		ylabel('y (m)') ;
		axis equal
		grid on
		pause(0)

		if (iteration ~= 0)
			M(i) = getframe;
		end
                
	end

	if (iteration ~= 0)
		movie(M, iteration, fps)
	end

elseif option == 20 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

	% vector plot of full velocity shear field

	psi_bar = zeros(N_rad, N_azim, N_layer) ;
	for i = 1 : N_rad
		for k = 1 : N_layer
			psi_bar(i, :, k) = 0.5*delta_omega(k)*r(i)*r(i) ;
		end
	end
                
	for i = 1 : length(inds)
                        
		psi_primed = diagnostic_read(psi_filenames(i, :)) ;
        
		[dpsi_dtheta_1, dpsi_dr_1] = gradient_imp(psi_primed(:, :, 1) + psi_bar(:, :, 1), delta_theta, delta_r) ;
		[dpsi_dtheta_2, dpsi_dr_2] = gradient_imp(psi_primed(:, :, 2) + psi_bar(:, :, 2), delta_theta, delta_r) ;
		u_r = -dpsi_dtheta_1./r_mat + dpsi_dtheta_2./r_mat;
		u_theta = dpsi_dr_1 - dpsi_dr_2 ;
		u_x = u_r.*x_mat./sqrt(x_mat.*x_mat+y_mat.*y_mat) - u_theta.*y_mat./sqrt(x_mat.*x_mat+y_mat.*y_mat) ;
		u_y = u_r.*y_mat./sqrt(x_mat.*x_mat+y_mat.*y_mat) + u_theta.*x_mat./sqrt(x_mat.*x_mat+y_mat.*y_mat) ;
                
		quiver(x_mat(:,:), y_mat(:,:), u_x(:,:), u_y(:,:), 'k')
		axis([-1.1*b, 1.1*b, -1.1*b, 1.1*b])
		text(-1.2*b, -b, ['[\Delta u_r , \Delta u_{theta} ] (r, \theta, t=', num2str(inds(i)*delta_t), 's)'], 'Rotation', 90) ;
		title([wd, '   (F, d) = (', num2str(F), ', ', num2str(d), ')']) ;
		xlabel('x (m)') ;
		ylabel('y (m)') ;
		axis equal
		grid on
		pause(0)
                
		if (iteration ~= 0)
			M(i) = getframe;
		end

	end

	if (iteration ~= 0)
		movie(M, iteration, fps)
	end

elseif option == 21 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

	% time-series of 1/4 and 3/4-radius interface height wave amplitude (neglecting tension)

	amp1_4 = [] ;
	amp3_4 = [] ;
	for i = 1 : length(inds)
		psi_primed = diagnostic_read(psi_filenames(i,:)) ;
		amp1_4 = [amp1_4, f/g_primed*(max(psi_primed(floor(N_rad/4),:,2)-psi_primed(floor(N_rad/4),:,1)) ...
			- min(psi_primed(floor(N_rad/4),:,2)-psi_primed(floor(N_rad/4),:,1)))/2.0] ;
		amp3_4 = [amp3_4, f/g_primed*(max(psi_primed(floor(3*N_rad/4),:,2)-psi_primed(floor(3*N_rad/4),:,1)) ...
			- min(psi_primed(floor(3*N_rad/4),:,2)-psi_primed(floor(3*N_rad/4),:,1)))/2.0] ;
	end
                   
	t = inds*delta_t ;
           
	plot(t, 1000*amp1_4, 'x') ;
	hold on;
	plot(t, 1000*amp3_4, '+') ;
	ylabel(['amplitude of \eta '' (r=fixed, \theta, t=fixed)   (mm)']) ;
	xlabel('t (s)') ;
	title([wd, '   (F, d) = (', num2str(F), ', ', num2str(d), ')']) ;
	grid on ;
	legend('r = one-quarter radius', 'r = three-quarter radius') ;

elseif option == 22 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

	% contour plot of local Richardson number in interfacial Ekman layer

	psi_bar = zeros(N_rad, N_azim, N_layer) ;
	for i = 1 : N_rad
		for k = 1 : N_layer
			psi_bar(i, :, k) = 0.5*delta_omega(k)*r(i)*r(i) ;
		end
	end     

	x_mat(:, N_azim+1) = x_mat(:, 1) ;                              % } to join up data across final azimuthal gridpoint
	y_mat(:, N_azim+1) = y_mat(:, 1) ;                              % }

	interfacial_Ekman_layer_depth = 2*sqrt(mean(new)/omega) ;

	for i = 1 : length(inds)
                
		psi_primed = diagnostic_read(psi_filenames(i, :)) ;
                
		[dpsi_dtheta_1, dpsi_dr_1] = gradient_imp(psi_primed(:, :, 1) + psi_bar(:, :, 1), delta_theta, delta_r) ;
		[dpsi_dtheta_2, dpsi_dr_2] = gradient_imp(psi_primed(:, :, 2) + psi_bar(:, :, 2), delta_theta, delta_r) ;
		u_r = -dpsi_dtheta_1./r_mat + dpsi_dtheta_2./r_mat;	% radial component of shear (top layer - bottom layer)
		u_theta = dpsi_dr_1 - dpsi_dr_2 ;			% azimuthal component of shear (top layer - bottom layer)

		Ri = g_primed * interfacial_Ekman_layer_depth ./ ( u_r .* u_r + u_theta .* u_theta ) ;

		Ri(:, N_azim+1) = Ri(:, 1) ;      % to join up data across final azimuthal gridpoint

		contourf(x_mat(:,:), y_mat(:,:), Ri(:, :), 20) ;
		shading flat
		colorbar_handle = colorbar ;
		set(get(colorbar_handle,'YLabel'),'String', ...
			['Ri (r, \theta, t=', num2str(inds(i)*delta_t), 's)']) ;
		title([wd, '   (F, d) = (', num2str(F), ', ', num2str(d), ')']) ;
		axis([-1.1*b, 1.1*b, -1.1*b, 1.1*b])
		xlabel('x (m)') ;
		ylabel('y (m)') ;
		axis equal
		grid on
		pause(0)

		if (iteration ~= 0)
			M(i) = getframe;
		end

	end
                        
	if (iteration ~= 0)
		movie(M, iteration, fps)
	end

elseif option == 23 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

	% time series of minimum local Richardson number in interfacial Ekman layer

	psi_bar = zeros(N_rad, N_azim, N_layer) ;
	for i = 1 : N_rad
		for k = 1 : N_layer
			psi_bar(i, :, k) = 0.5*delta_omega(k)*r(i)*r(i) ;
		end
	end

	interfacial_Ekman_layer_depth = 2*sqrt(mean(new)/omega) ;

	min_Ri = [] ;
	for i = 1 : length(inds)
		psi_primed = diagnostic_read(psi_filenames(i, :)) ;
		[dpsi_dtheta_1, dpsi_dr_1] = gradient_imp(psi_primed(:, :, 1) + psi_bar(:, :, 1), delta_theta, delta_r) ;
		[dpsi_dtheta_2, dpsi_dr_2] = gradient_imp(psi_primed(:, :, 2) + psi_bar(:, :, 2), delta_theta, delta_r) ;
		u_r = -dpsi_dtheta_1./r_mat + dpsi_dtheta_2./r_mat;     % radial component of shear (top layer - bottom layer)
		u_theta = dpsi_dr_1 - dpsi_dr_2 ;                       % azimuthal component of shear (top layer - bottom layer)
		Ri = g_primed * interfacial_Ekman_layer_depth ./ ( u_r .* u_r + u_theta .* u_theta ) ;
		min_Ri = [min_Ri, min(min(Ri))] ;
	end

	t = inds*delta_t ;

	plot(t, min_Ri, 'x') ;
	ylabel(['minimum Ri (r, \theta, t=fixed)']) ;
	xlabel('t (s)') ;
	title([wd, '   (F, d) = (', num2str(F), ', ', num2str(d), ')']) ;
	grid on ;

	hold on ;
	plot([min(t) max(t)], [8 * g_primed * sqrt(mean(new)) / (b^2 * lid_delta_omega^2 * sqrt(omega)) ...
				8 * g_primed * sqrt(mean(new)) / (b^2 * lid_delta_omega^2 * sqrt(omega))], '--') ;
        fprintf(1, ['\n     minimum local Richardson number for the equilibrium flow is: ' ...
				num2str(8 * g_primed * sqrt(mean(new)) / (b^2 * lid_delta_omega^2 * sqrt(omega)))]) ;

elseif option == 24 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

	% contour plot of turbulent energy dissipation rate (epsilon)

	psi_bar = zeros(N_rad, N_azim, N_layer) ;
	for i = 1 : N_rad
		for k = 1 : N_layer
			psi_bar(i, :, k) = 0.5*delta_omega(k)*r(i)*r(i) ;
		end
	end

	x_mat(:, N_azim+1) = x_mat(:, 1) ;                              % } to join up data across final azimuthal gridpoint   
	y_mat(:, N_azim+1) = y_mat(:, 1) ;                              % }

	interfacial_Ekman_layer_depth = 2*sqrt(mean(new)/omega) ;

	for i = 1 : length(inds)

		% get Ri at time i-1
		if i == 1
			tmp_PSI_ = ['PSI_.0000000000'] ;
			tmp_PSI_(16-length(num2str(start-increment)) : 15) = num2str(start-increment) ;
			psi_primed = diagnostic_read(tmp_PSI_) ;
		else
			psi_primed = diagnostic_read(psi_filenames(i-1, :)) ;
		end
		[dpsi_dtheta_1, dpsi_dr_1] = gradient_imp(psi_primed(:, :, 1) + psi_bar(:, :, 1), delta_theta, delta_r) ;
		[dpsi_dtheta_2, dpsi_dr_2] = gradient_imp(psi_primed(:, :, 2) + psi_bar(:, :, 2), delta_theta, delta_r) ;
		u_r = -dpsi_dtheta_1./r_mat + dpsi_dtheta_2./r_mat;	% radial component of shear (top layer - bottom layer)
		u_theta = dpsi_dr_1 - dpsi_dr_2 ;			% azimuthal component of shear (top layer - bottom layer)
		Ri_minus = g_primed * interfacial_Ekman_layer_depth ./ ( u_r .* u_r + u_theta .* u_theta ) ;
                Ri_minusx3 = filter(ones(1, windowSize)/windowSize, 1, [Ri_minus, Ri_minus, Ri_minus], [], 2) ;
                Ri_minus = Ri_minusx3(:, N_azim+1+floor(windowSize/2) : 2*N_azim+floor(windowSize/2)) ;

		% get Ri at time i
		psi_primed = diagnostic_read(psi_filenames(i, :)) ;
		[dpsi_dtheta_1, dpsi_dr_1] = gradient_imp(psi_primed(:, :, 1) + psi_bar(:, :, 1), delta_theta, delta_r) ;
		[dpsi_dtheta_2, dpsi_dr_2] = gradient_imp(psi_primed(:, :, 2) + psi_bar(:, :, 2), delta_theta, delta_r) ;
		u_r = -dpsi_dtheta_1./r_mat + dpsi_dtheta_2./r_mat;	% radial component of shear (top layer - bottom layer)
		u_theta = dpsi_dr_1 - dpsi_dr_2 ;			% azimuthal component of shear (top layer - bottom layer)
		Ri_now = g_primed * interfacial_Ekman_layer_depth ./ ( u_r .* u_r + u_theta .* u_theta ) ;
		[dpsi_dtheta_now, dpsi_dr_now] = gradient_imp(psi_primed(:, :, layer) + psi_bar(:, :, layer), delta_theta, delta_r) ;
		delta_u_squared = u_r .* u_r + u_theta .* u_theta ;	% for use when calculating epsilon
                Ri_nowx3 = filter(ones(1, windowSize)/windowSize, 1, [Ri_now, Ri_now, Ri_now], [], 2) ;
                Ri_now = Ri_nowx3(:, N_azim+1+floor(windowSize/2) : 2*N_azim+floor(windowSize/2)) ;

		% get Ri at time i+1
		if i == length(inds)
			tmp_PSI_ = ['PSI_.0000000000'] ;
			tmp_PSI_(16-length(num2str(finish+increment)) : 15) = num2str(finish+increment) ;
			psi_primed = diagnostic_read(tmp_PSI_) ;
		else
			psi_primed = diagnostic_read(psi_filenames(i+1, :)) ;
		end
		[dpsi_dtheta_1, dpsi_dr_1] = gradient_imp(psi_primed(:, :, 1) + psi_bar(:, :, 1), delta_theta, delta_r) ;
		[dpsi_dtheta_2, dpsi_dr_2] = gradient_imp(psi_primed(:, :, 2) + psi_bar(:, :, 2), delta_theta, delta_r) ;
		u_r = -dpsi_dtheta_1./r_mat + dpsi_dtheta_2./r_mat;     % radial component of shear (top layer - bottom layer)
		u_theta = dpsi_dr_1 - dpsi_dr_2 ;                       % azimuthal component of shear (top layer - bottom layer)
		Ri_plus = g_primed * interfacial_Ekman_layer_depth ./ ( u_r .* u_r + u_theta .* u_theta ) ;
                Ri_plusx3 = filter(ones(1, windowSize)/windowSize, 1, [Ri_plus, Ri_plus, Ri_plus], [], 2) ;
                Ri_plus = Ri_plusx3(:, N_azim+1+floor(windowSize/2) : 2*N_azim+floor(windowSize/2)) ;
           
		% calculate phi using D/Dt = d/dt + u.grad
		phi = - (log(Ri_plus) - log(Ri_minus)) / (2*delta_t*increment) ;				% local rate of change
		[dlnRi_dtheta_now, dlnRi_dr_now] = gradient_imp(log(Ri_now), delta_theta, delta_r) ;
		phi = - ( - dpsi_dtheta_now .* dlnRi_dr_now + dpsi_dr_now .* dlnRi_dtheta_now ) ./ r_mat ;      % advection in layer

		% calculate epsilon
		epsilon = phi .* delta_u_squared / 24.0 ;
		epsilon = 0.5 * (epsilon + epsilon .* sign(epsilon)) ;			% to set negative values to zero

		% plot epsilon
		epsilon(:, N_azim+1) = epsilon(:, 1) ;
		contourf(x_mat(:,:), y_mat(:,:), epsilon(:, :), 20) ;
		shading flat
		colorbar_handle = colorbar ;
		set(get(colorbar_handle,'YLabel'),'String', ...
			['\epsilon (r, \theta, layer=', num2str(layer), ', t=', num2str(inds(i)*delta_t), 's)   (m^2 s^-^3)']) ;
		title([wd, '   (F, d) = (', num2str(F), ', ', num2str(d), ')']) ;
		axis([-1.1*b, 1.1*b, -1.1*b, 1.1*b])
		xlabel('x (m)') ;
		ylabel('y (m)') ;
		axis equal
		grid on
		pause(0)

		if (iteration ~= 0)
			M(i) = getframe;
		end
           
	end

	if (iteration ~= 0)
		movie(M, iteration, fps)
	end

elseif option == 25 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                
	% interface height profiles as a function of theta (neglecting interfacial tension and centripetal effects)
                   
	eta_bar = zeros(N_rad, N_azim) ;
	for i = 1 : N_rad
		eta_bar(i, :) = omega*omega*r(i)*r(i)*(1.0 - (lid_delta_omega/omega)/(delta_rho/rho_bar))/(2.0*g) ;
	end
               
	for i = 1 : length(inds)
        
		psi_primed = diagnostic_read(psi_filenames(i, :)) ;
		eta = eta_bar + f/g_primed*(psi_primed(:, :, 2) - psi_primed(:, :, 1)) ;
		eta = eta - mean(mean(eta .* r_mat .* area_factor)) / mean(mean(r_mat .* area_factor)) ;	% subtract <eta> to ensure volume conservation
		eta_filteredx3 = filter(ones(1, windowSize)/windowSize, 1, [eta,eta,eta], [], 2) ;
		eta = eta_filteredx3(:, N_azim+1+floor(windowSize/2) : 2*N_azim+floor(windowSize/2)) ;

		plot(theta(:)*180/pi, 1000*(H+eta(:,:)), 'LineWidth', 2) ;
		xlabel('\theta (degrees)') ;
		ylabel(['h (r, \theta, t=', num2str(inds(i)*delta_t), 's)   (mm)']) ;
		title([wd, '   (F, d) = (', num2str(F), ', ', num2str(d), ')']) ;
		set(gca, 'Xlim', [0,360]) ;
		set(gca, 'Xtick', 0:60:360) ;
		grid on
		
		for j=1:N_rad
			r_eq(j,:) = 'r = ' ;
			mm(j,:) = ' mm' ;
		end
		legend([r_eq, num2str(round(r'*1000)), mm], 4)
		
		pause(0)
		if (iteration ~= 0)
			M(i) = getframe;
		end
                
	end
                
	if (iteration ~= 0)
		movie(M, iteration, fps)
	end

elseif option == 26 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

	% time series of system energy

	psi_bar = zeros(N_rad, N_azim, N_layer) ;
	for i = 1 : N_rad  
		for k = 1 : N_layer
			psi_bar(i, :, k) = 0.5*delta_omega(k)*r(i)*r(i) ;
		end     
	end

	eta_bar = zeros(N_rad, N_azim) ;
	for i = 1 : N_rad
		eta_bar(i, :) = omega*omega*r(i)*r(i)*(1.0 - (lid_delta_omega/omega)/(delta_rho/rho_bar))/(2.0*g) ;  
	end

	% set up area element matrix
	dA = r_mat * delta_r * delta_theta ;
	dA(1, :) = dA(1, :)/2 ;
	dA(N_rad, :) = dA(N_rad, :)/2 ;

	GPE = [] ;
	KE = [] ;

	for i = 1 : length(inds)

                psi_primed = diagnostic_read(psi_filenames(i, :)) ;
                eta = eta_bar + f/g_primed*(psi_primed(:, :, 2) - psi_primed(:, :, 1)) ;
                eta = eta - mean(mean(eta .* r_mat .* area_factor)) / mean(mean(r_mat .* area_factor)) ;      % subtract <eta> to ensure volume conservation
                GPE = [GPE, 0.5 * g * delta_rho * sum(sum(eta .* eta .* dA))] ;

                [dpsi_dtheta_1, dpsi_dr_1] = gradient_imp(psi_primed(:, :, 1) + psi_bar(:, :, 1), delta_theta, delta_r) ;
                u_r_1 = -dpsi_dtheta_1./r_mat ;
                u_theta_1 = dpsi_dr_1 ;
		[dpsi_dtheta_2, dpsi_dr_2] = gradient_imp(psi_primed(:, :, 2) + psi_bar(:, :, 2), delta_theta, delta_r) ;
		u_r_2 = -dpsi_dtheta_2./r_mat ;  
		u_theta_2 = dpsi_dr_2 ;
		KE = [KE, sum(sum(     0.5 * rho(2) * (H+eta) .* (u_r_2.*u_r_2 + u_theta_2.*u_theta_2) .* dA ...
				     + 0.5 * rho(1) * (H-eta) .* (u_r_1.*u_r_1 + u_theta_1.*u_theta_1) .* dA      ))  ] ;

	end

	t = inds*delta_t ;

	plot(t, KE, 'x') ;
	hold on;
	plot(t, 10*GPE, '+r') ;
	plot(t, GPE+KE, '*') ;
	ylabel('E (t)   (J)') ;
	xlabel('t (s)') ;
	title([wd, '   (F, d) = (', num2str(F), ', ', num2str(d), ')']) ;
	grid on ;
	legend('kinetic energy', '10*potential energy', 'total energy', 0);

elseif option == 27 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

	% 3-D image of full interface height (neglecting interfacial tension and centripetal effects)

	x_mat(:, N_azim+1) = x_mat(:, 1) ;                              % } to join up data across final azimuthal gridpoint
	y_mat(:, N_azim+1) = y_mat(:, 1) ;                              % }
                
	eta_bar = zeros(N_rad, N_azim) ;
	for i = 1 : N_rad
		eta_bar(i, :) = omega*omega*r(i)*r(i)*(1.0 - (lid_delta_omega/omega)/(delta_rho/rho_bar))/(2.0*g) ;
	end
                
	for i = 1 : length(inds) 
                
		psi_primed = diagnostic_read(psi_filenames(i, :)) ;
		eta = eta_bar + f/g_primed*(psi_primed(:, :, 2) - psi_primed(:, :, 1)) ;
		eta = eta - mean(mean(eta .* r_mat .* area_factor)) / mean(mean(r_mat .* area_factor)) ;      % subtract <eta> to ensure volume conservation
                        
		eta(:, N_azim+1) = eta(:, 1) ;                          % to join up data across final azimuthal gridpoint
		surf(x_mat(:,:), y_mat(:,:), 1000*(H+eta(:,:))) ;
%		surf(x_mat(2:N_rad-1,:), y_mat(2:N_rad-1,:), 1000*(H+eta(2:N_rad-1,:))) ;
%		axis([-b, b, -b, b, 0, 1000*N_layer*H]) ;
%		shading flat
		title([wd, '   (F, d) = (', num2str(F), ', ', num2str(d), ')']) ;
		xlabel('x (m)') ;
		ylabel('y (m)') ;
		zlabel(['z = h (r, \theta, t=', num2str(inds(i)*delta_t), 's)   (mm)']) ;
		grid on
		pause(0)
		if (iteration ~= 0)
			M(i) = getframe;
		end

%		fprintf(1, 'WARNING: save jpeg images switched on!\n') ;
%		print( gcf, '-r50', '-djpeg90', ['movie', num2str(inds(i))] ) ;	% number following r is resolution in dpi (default 150)
										% number following djpeg is quality level (default 75)

										% then to create movie use convert -delay 50 -loop 0 movie* movie.gif
										% number following delay is delay between frames in 1/100 s (default 0)
										% number following loop is number of movie iterations (default 1, 0=>infinite)

	end
                
	if (iteration ~= 0)
		movie(M, iteration, fps)
	end

elseif option == 28 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

	% multiple radiation diagnostics

	% PREPARATIONS
        x_mat(:, N_azim+1) = x_mat(:, 1) ;                              % } to join up data across final azimuthal gridpoint
        y_mat(:, N_azim+1) = y_mat(:, 1) ;                              % }
        eta_bar = zeros(N_rad, N_azim) ;
        for i = 1 : N_rad
                eta_bar(i, :) = omega*omega*r(i)*r(i)*(1.0 - (lid_delta_omega/omega)/(delta_rho/rho_bar))/(2.0*g) ;
        end
        psi_bar = zeros(N_rad, N_azim, N_layer) ;
        for i = 1 : N_rad
                for k = 1 : N_layer
                        psi_bar(i, :, k) = 0.5*delta_omega(k)*r(i)*r(i) ;   
                end
        end
	interfacial_Ekman_layer_depth = 2*sqrt(mean(new)/omega) ;


        for i = 1 : length(inds)

		% OPTION 7
		subplot(3,2,1) ;           
		set(gcf, 'Position', [200 20 800 650]) ;
		set(gcf, 'PaperPosition', [0.35 0.76 8.8 11]) ;
                psi_primed = diagnostic_read(psi_filenames(i, :)) ;
                eta = eta_bar + f/g_primed*(psi_primed(:, :, 2) - psi_primed(:, :, 1)) ;
                eta = eta - mean(mean(eta .* r_mat .* area_factor)) / mean(mean(r_mat .* area_factor)) ;        % subtract <eta> to ensure volume conservation
                eta(:, N_azim+1) = eta(:, 1) ;                          % to join up data across final azimuthal gridpoint
                contourf(x_mat(:,:), y_mat(:,:), 1000*(H+eta), 20) ;
                shading flat
                colorbar_handle = colorbar ;
                set(get(colorbar_handle,'YLabel'),'String', ...
                        ['h (r, \theta, t=', num2str(inds(i)*delta_t), 's)   (mm)']) ;
                title([wd, '   (F, d) = (', num2str(F), ', ', num2str(d), ')']) ;
                xlabel('x (m)') ;
                ylabel('y (m)') ;
                axis equal
		axis([-1.1*b, 1.1*b, -1.1*b, 1.1*b])
                grid on

%		% velocity shear contour plot
%		subplot(3,2,2) ;
%                [dpsi_dtheta_1, dpsi_dr_1] = gradient_imp(psi_primed(:, :, 1) + psi_bar(:, :, 1), delta_theta, delta_r) ;
%                [dpsi_dtheta_2, dpsi_dr_2] = gradient_imp(psi_primed(:, :, 2) + psi_bar(:, :, 2), delta_theta, delta_r) ;
%                u_r = -dpsi_dtheta_1./r_mat + dpsi_dtheta_2./r_mat;
%                u_theta = dpsi_dr_1 - dpsi_dr_2 ;
%		u_r(:, N_azim+1) = u_r(:, 1) ;
%		u_theta(:, N_azim+1) = u_theta(:, 1) ;
%		contourf(x_mat, y_mat, 1000*sqrt(u_r.*u_r+u_theta.*u_theta), 20) ;
%                axis([-1.1*b, 1.1*b, -1.1*b, 1.1*b])
%		shading flat
%		colorbar_handle = colorbar ;
%		set(get(colorbar_handle,'YLabel'),'String', ...
%			['|(\Delta u_r , \Delta u_{theta})| (r, \theta, t=', num2str(inds(i)*delta_t), 's)   (mm s^-^1)']) ;
%                title([wd, '   (F, d) = (', num2str(F), ', ', num2str(d), ')']) ;
%                xlabel('x (m)') ;
%                ylabel('y (m)') ;
%                axis equal
%                grid on

		% OPTION 29
		subplot(3,2,2) ;
		% get eta at time i-1
		if i == 1
			tmp_PSI_ = ['PSI_.0000000000'] ;
			tmp_PSI_(16-length(num2str(start-increment)) : 15) = num2str(start-increment) ;
			psi_primed = diagnostic_read(tmp_PSI_) ;
		else
			psi_primed = diagnostic_read(psi_filenames(i-1, :)) ;
		end
		eta_minus = eta_bar + f/g_primed*(psi_primed(:, :, 2) - psi_primed(:, :, 1)) ;
		eta_minus = eta_minus - mean(mean(eta_minus .* r_mat .* area_factor)) / mean(mean(r_mat .* area_factor)) ;	% volume cons.
		% get eta at time i
		psi_primed = diagnostic_read(psi_filenames(i, :)) ;
                eta_now = eta_bar + f/g_primed*(psi_primed(:, :, 2) - psi_primed(:, :, 1)) ;
                eta_now = eta_now - mean(mean(eta_now .* r_mat .* area_factor)) / mean(mean(r_mat .* area_factor)) ;		% volume cons. 
		[dpsi_dtheta_now, dpsi_dr_now] = gradient_imp(psi_primed(:, :, layer) + psi_bar(:, :, layer), delta_theta, delta_r) ;
		% get eta at time i+1
		if i == length(inds)
			tmp_PSI_ = ['PSI_.0000000000'] ;
			tmp_PSI_(16-length(num2str(finish+increment)) : 15) = num2str(finish+increment) ;
			psi_primed = diagnostic_read(tmp_PSI_) ;
		else
			psi_primed = diagnostic_read(psi_filenames(i+1, :)) ;
		end
                eta_plus = eta_bar + f/g_primed*(psi_primed(:, :, 2) - psi_primed(:, :, 1)) ;
                eta_plus = eta_plus - mean(mean(eta_plus .* r_mat .* area_factor)) / mean(mean(r_mat .* area_factor)) ;		% volume cons. 
		% calculate delta using D/Dt = d/dt + u.grad
		derivative = (eta_plus - eta_minus) / (2*delta_t*increment) ;				% local rate of change
		[deta_dtheta_now, deta_dr_now] = gradient_imp(eta_now, delta_theta, delta_r) ;
		derivative = derivative + ( - dpsi_dtheta_now .* deta_dr_now + dpsi_dr_now .* deta_dtheta_now ) ./ r_mat ;	% advection in a layer
		if (layer == 1)
			delta = derivative ./ (H - eta_now);
		elseif (layer == 2)
			delta = - derivative ./ (H + eta_now);
		end
                % filter to remove spurious noise
                deltax3 = filter(ones(1, windowSize)/windowSize, 1, [delta, delta, delta], [], 2) ;
                delta = deltax3(:, N_azim+1+floor(windowSize/2) : 2*N_azim+floor(windowSize/2)) ;  
		% plot delta
		delta(:, N_azim+1) = delta(:, 1) ;
		contourf(x_mat(:,:), y_mat(:,:), delta(:, :), 20) ;
		shading flat
		colorbar_handle = colorbar ;
		set(get(colorbar_handle,'YLabel'),'String', ...
			['\delta (r, \theta, layer=', num2str(layer), ', t=', num2str(inds(i)*delta_t), 's)   (s^-^1)']) ;
		title([wd, '   (F, d) = (', num2str(F), ', ', num2str(d), ')']) ;
		axis([-1.1*b, 1.1*b, -1.1*b, 1.1*b])
		xlabel('x (m)') ;
		ylabel('y (m)') ;
		axis equal
		axis([-1.1*b, 1.1*b, -1.1*b, 1.1*b])
		grid on

		% OPTION 22
		subplot(3, 2, 3) ;
		psi_primed = diagnostic_read(psi_filenames(i, :)) ;
                [dpsi_dtheta_1, dpsi_dr_1] = gradient_imp(psi_primed(:, :, 1) + psi_bar(:, :, 1), delta_theta, delta_r) ;
                [dpsi_dtheta_2, dpsi_dr_2] = gradient_imp(psi_primed(:, :, 2) + psi_bar(:, :, 2), delta_theta, delta_r) ;
                u_r = -dpsi_dtheta_1./r_mat + dpsi_dtheta_2./r_mat;     % radial component of shear (top layer - bottom layer)
                u_theta = dpsi_dr_1 - dpsi_dr_2 ;                       % azimuthal component of shear (top layer - bottom layer)
                Ri = g_primed * interfacial_Ekman_layer_depth ./ ( u_r .* u_r + u_theta .* u_theta ) ;
                Ri(:, N_azim+1) = Ri(:, 1) ;      % to join up data across final azimuthal gridpoint
                contourf(x_mat(:,:), y_mat(:,:), Ri(:, :), 20) ;
                shading flat
                colorbar_handle = colorbar ;
                set(get(colorbar_handle,'YLabel'),'String', ...
                        ['Ri (r, \theta, t=', num2str(inds(i)*delta_t), 's)']) ;
                title([wd, '   (F, d) = (', num2str(F), ', ', num2str(d), ')']) ;
                axis([-1.1*b, 1.1*b, -1.1*b, 1.1*b])
                xlabel('x (m)') ;
                ylabel('y (m)') ;
                axis equal
		axis([-1.1*b, 1.1*b, -1.1*b, 1.1*b])
                grid on

		% OPTION 19       
		subplot(3, 2, 4);
                % get Ri at time i-1
                if i == 1
                        tmp_PSI_ = ['PSI_.0000000000'] ;
                        tmp_PSI_(16-length(num2str(start-increment)) : 15) = num2str(start-increment) ;
                        psi_primed = diagnostic_read(tmp_PSI_) ;
                else
                        psi_primed = diagnostic_read(psi_filenames(i-1, :)) ;  
                end
                [dpsi_dtheta_1, dpsi_dr_1] = gradient_imp(psi_primed(:, :, 1) + psi_bar(:, :, 1), delta_theta, delta_r) ;
                [dpsi_dtheta_2, dpsi_dr_2] = gradient_imp(psi_primed(:, :, 2) + psi_bar(:, :, 2), delta_theta, delta_r) ;
                u_r = -dpsi_dtheta_1./r_mat + dpsi_dtheta_2./r_mat;     % radial component of shear (top layer - bottom layer)
                u_theta = dpsi_dr_1 - dpsi_dr_2 ;                       % azimuthal component of shear (top layer - bottom layer)
                Ri_minus = g_primed * interfacial_Ekman_layer_depth ./ ( u_r .* u_r + u_theta .* u_theta ) ;
                Ri_minusx3 = filter(ones(1, windowSize)/windowSize, 1, [Ri_minus, Ri_minus, Ri_minus], [], 2) ;
                Ri_minus = Ri_minusx3(:, N_azim+1+floor(windowSize/2) : 2*N_azim+floor(windowSize/2)) ;
                % get Ri at time i
                psi_primed = diagnostic_read(psi_filenames(i, :)) ;
                [dpsi_dtheta_1, dpsi_dr_1] = gradient_imp(psi_primed(:, :, 1) + psi_bar(:, :, 1), delta_theta, delta_r) ;
                [dpsi_dtheta_2, dpsi_dr_2] = gradient_imp(psi_primed(:, :, 2) + psi_bar(:, :, 2), delta_theta, delta_r) ;   
                u_r = -dpsi_dtheta_1./r_mat + dpsi_dtheta_2./r_mat;     % radial component of shear (top layer - bottom layer)
                u_theta = dpsi_dr_1 - dpsi_dr_2 ;                       % azimuthal component of shear (top layer - bottom layer)
                Ri_now = g_primed * interfacial_Ekman_layer_depth ./ ( u_r .* u_r + u_theta .* u_theta ) ;
                [dpsi_dtheta_now, dpsi_dr_now] = gradient_imp(psi_primed(:, :, layer) + psi_bar(:, :, layer), delta_theta, delta_r) ;
                u_r_now = -dpsi_dtheta_now./r_mat ;
                u_theta_now = dpsi_dr_now ;
                Ri_nowx3 = filter(ones(1, windowSize)/windowSize, 1, [Ri_now, Ri_now, Ri_now], [], 2) ;
                Ri_now = Ri_nowx3(:, N_azim+1+floor(windowSize/2) : 2*N_azim+floor(windowSize/2)) ;
                % get Ri at time i+1
                if i == length(inds)
                        tmp_PSI_ = ['PSI_.0000000000'] ;
                        tmp_PSI_(16-length(num2str(finish+increment)) : 15) = num2str(finish+increment) ;
                        psi_primed = diagnostic_read(tmp_PSI_) ;
                else
                        psi_primed = diagnostic_read(psi_filenames(i+1, :)) ;
                end
                [dpsi_dtheta_1, dpsi_dr_1] = gradient_imp(psi_primed(:, :, 1) + psi_bar(:, :, 1), delta_theta, delta_r) ;
                [dpsi_dtheta_2, dpsi_dr_2] = gradient_imp(psi_primed(:, :, 2) + psi_bar(:, :, 2), delta_theta, delta_r) ;
                u_r = -dpsi_dtheta_1./r_mat + dpsi_dtheta_2./r_mat;     % radial component of shear (top layer - bottom layer)
                u_theta = dpsi_dr_1 - dpsi_dr_2 ;                       % azimuthal component of shear (top layer - bottom layer)
                Ri_plus = g_primed * interfacial_Ekman_layer_depth ./ ( u_r .* u_r + u_theta .* u_theta ) ;
                Ri_plusx3 = filter(ones(1, windowSize)/windowSize, 1, [Ri_plus, Ri_plus, Ri_plus], [], 2) ;
                Ri_plus = Ri_plusx3(:, N_azim+1+floor(windowSize/2) : 2*N_azim+floor(windowSize/2)) ;
                % calculate phi using D/Dt = d/dt + u.grad
                phi = - (log(Ri_plus) - log(Ri_minus)) / (2*delta_t*increment) ;                                % local rate of change
                [dlnRi_dtheta_now, dlnRi_dr_now] = gradient_imp(log(Ri_now), delta_theta, delta_r) ;
                phi = phi - ( - dpsi_dtheta_now .* dlnRi_dr_now + dpsi_dr_now .* dlnRi_dtheta_now ) ./ r_mat ;  % advection in a layer
                % plot phi
                phi(:, N_azim+1) = phi(:, 1) ;
                contourf(x_mat(:,:), y_mat(:,:), phi(:, :), 20) ;
                shading flat
                colorbar_handle = colorbar ;
                set(get(colorbar_handle,'YLabel'),'String', ...
                        ['\phi (r, \theta, layer=', num2str(layer), ', t=', num2str(inds(i)*delta_t), 's)   (s^-^1)']) ;
                title([wd, '   (F, d) = (', num2str(F), ', ', num2str(d), ')']) ;
                axis([-1.1*b, 1.1*b, -1.1*b, 1.1*b])
                xlabel('x (m)') ;
                ylabel('y (m)') ;
                axis equal
		axis([-1.1*b, 1.1*b, -1.1*b, 1.1*b])
                grid on
 
		% OPTION 24
		subplot(3,2,5);
                % get Ri at time i-1
                if i == 1
                        tmp_PSI_ = ['PSI_.0000000000'] ;
                        tmp_PSI_(16-length(num2str(start-increment)) : 15) = num2str(start-increment) ;
                        psi_primed = diagnostic_read(tmp_PSI_) ; 
                else    
                        psi_primed = diagnostic_read(psi_filenames(i-1, :)) ;
                end
                [dpsi_dtheta_1, dpsi_dr_1] = gradient_imp(psi_primed(:, :, 1) + psi_bar(:, :, 1), delta_theta, delta_r) ;
                [dpsi_dtheta_2, dpsi_dr_2] = gradient_imp(psi_primed(:, :, 2) + psi_bar(:, :, 2), delta_theta, delta_r) ;
                u_r = -dpsi_dtheta_1./r_mat + dpsi_dtheta_2./r_mat;     % radial component of shear (top layer - bottom layer)
                u_theta = dpsi_dr_1 - dpsi_dr_2 ;                       % azimuthal component of shear (top layer - bottom layer)
                Ri_minus = g_primed * interfacial_Ekman_layer_depth ./ ( u_r .* u_r + u_theta .* u_theta ) ;
                Ri_minusx3 = filter(ones(1, windowSize)/windowSize, 1, [Ri_minus, Ri_minus, Ri_minus], [], 2) ;
                Ri_minus = Ri_minusx3(:, N_azim+1+floor(windowSize/2) : 2*N_azim+floor(windowSize/2)) ;
                % get Ri at time i
                psi_primed = diagnostic_read(psi_filenames(i, :)) ;
                [dpsi_dtheta_1, dpsi_dr_1] = gradient_imp(psi_primed(:, :, 1) + psi_bar(:, :, 1), delta_theta, delta_r) ;
                [dpsi_dtheta_2, dpsi_dr_2] = gradient_imp(psi_primed(:, :, 2) + psi_bar(:, :, 2), delta_theta, delta_r) ;   
                u_r = -dpsi_dtheta_1./r_mat + dpsi_dtheta_2./r_mat;     % radial component of shear (top layer - bottom layer)
                u_theta = dpsi_dr_1 - dpsi_dr_2 ;                       % azimuthal component of shear (top layer - bottom layer)
                Ri_now = g_primed * interfacial_Ekman_layer_depth ./ ( u_r .* u_r + u_theta .* u_theta ) ;
                [dpsi_dtheta_now, dpsi_dr_now] = gradient_imp(psi_primed(:, :, layer) + psi_bar(:, :, layer), delta_theta, delta_r) ;
                delta_u_squared = u_r .* u_r + u_theta .* u_theta ;     % for use when calculating epsilon
                Ri_nowx3 = filter(ones(1, windowSize)/windowSize, 1, [Ri_now, Ri_now, Ri_now], [], 2) ;
                Ri_now = Ri_nowx3(:, N_azim+1+floor(windowSize/2) : 2*N_azim+floor(windowSize/2)) ;
                % get Ri at time i+1
                if i == length(inds)
                        tmp_PSI_ = ['PSI_.0000000000'] ;
                        tmp_PSI_(16-length(num2str(finish+increment)) : 15) = num2str(finish+increment) ;
                        psi_primed = diagnostic_read(tmp_PSI_) ;
                else
                        psi_primed = diagnostic_read(psi_filenames(i+1, :)) ;
                end
                [dpsi_dtheta_1, dpsi_dr_1] = gradient_imp(psi_primed(:, :, 1) + psi_bar(:, :, 1), delta_theta, delta_r) ;
                [dpsi_dtheta_2, dpsi_dr_2] = gradient_imp(psi_primed(:, :, 2) + psi_bar(:, :, 2), delta_theta, delta_r) ;
                u_r = -dpsi_dtheta_1./r_mat + dpsi_dtheta_2./r_mat;     % radial component of shear (top layer - bottom layer)
                u_theta = dpsi_dr_1 - dpsi_dr_2 ;                       % azimuthal component of shear (top layer - bottom layer)
                Ri_plus = g_primed * interfacial_Ekman_layer_depth ./ ( u_r .* u_r + u_theta .* u_theta ) ;
                Ri_plusx3 = filter(ones(1, windowSize)/windowSize, 1, [Ri_plus, Ri_plus, Ri_plus], [], 2) ;
                Ri_plus = Ri_plusx3(:, N_azim+1+floor(windowSize/2) : 2*N_azim+floor(windowSize/2)) ;
                % calculate phi using D/Dt = d/dt + u.grad
                phi = - (log(Ri_plus) - log(Ri_minus)) / (2*delta_t*increment) ;                                % local rate of change
                [dlnRi_dtheta_now, dlnRi_dr_now] = gradient_imp(log(Ri_now), delta_theta, delta_r) ;
                phi = - ( - dpsi_dtheta_now .* dlnRi_dr_now + dpsi_dr_now .* dlnRi_dtheta_now ) ./ r_mat ;      % advection in layer
                % calculate epsilon
                epsilon = phi .* delta_u_squared / 24.0 ;
                epsilon = 0.5 * (epsilon + epsilon .* sign(epsilon)) ;                  % to set negative values to zero
                % plot epsilon
                epsilon(:, N_azim+1) = epsilon(:, 1) ;
                contourf(x_mat(:,:), y_mat(:,:), epsilon(:, :), 20) ;
                shading flat
                colorbar_handle = colorbar ;
                set(get(colorbar_handle,'YLabel'),'String', ... 
                        ['\epsilon (r, \theta, layer=', num2str(layer), ', t=', num2str(inds(i)*delta_t), 's)   (m^2 s^-^3)']) ;
                title([wd, '   (F, d) = (', num2str(F), ', ', num2str(d), ')']) ;
                axis([-1.1*b, 1.1*b, -1.1*b, 1.1*b])
                xlabel('x (m)') ;
                ylabel('y (m)') ;
                axis equal
		axis([-1.1*b, 1.1*b, -1.1*b, 1.1*b])
                grid on

		% OPTION 18
		subplot(3, 2, 6);
                % PREVIOUS TIME
                % read in data
                        if i == 1
                                tmp_PSI_ = ['PSI_.0000000000'] ;
                                tmp_PSI_(16-length(num2str(start-increment)) : 15) = num2str(start-increment) ;
                                psi_primed = diagnostic_read(tmp_PSI_) ;
                        else
                                psi_primed = diagnostic_read(psi_filenames(i-1, :)) ;
                        end
                psi_primed_filteredx3 = filter(ones(1, windowSize)/windowSize, 1, [psi_primed, psi_primed, psi_primed], [], 2) ;
                psi_primed = psi_primed_filteredx3(:, N_azim+1+floor(windowSize/2) : 2*N_azim+floor(windowSize/2), :) ;
                % calculate u_r and u_theta in layer 2
                        [dpsi_dtheta, dpsi_dr] = gradient_imp(psi_primed(:, :, 2) + psi_bar(:, :, 2), delta_theta, delta_r) ;
                        u_r = -dpsi_dtheta./r_mat ;
                        u_theta = dpsi_dr ;
                % calculate h
                        eta = eta_bar + f/g_primed*(psi_primed(:, :, 2) - psi_primed(:, :, 1)) ;
                        eta = eta - mean(mean(eta .* r_mat .* area_factor)) / mean(mean(r_mat .* area_factor)) ;     % subtract <eta> to ensure volume conservation
                        h = H + eta ;
                % calculate some gradients which are needed to calculate F
                        [useless, d_dr_of_hu_r] = gradient_imp(h.*u_r, delta_theta, delta_r) ;
                        [d_dtheta_of_hu_theta, useless] = gradient_imp(h.*u_theta, delta_theta, delta_r) ;
                        [d_dtheta_of_u_r, d_dr_of_u_r] = gradient_imp(u_r, delta_theta, delta_r) ;
                        [d_dtheta_of_u_theta, d_dr_of_u_theta] = gradient_imp(u_theta, delta_theta, delta_r) ;
                % calculate components of F
                        F_r = u_r .* ( h .* u_r ./ r_mat + d_dr_of_hu_r + d_dtheta_of_hu_theta ./ r_mat ) ...
                                + h .* ( u_r .* d_dr_of_u_r + u_theta ./ r_mat .* d_dtheta_of_u_r - u_theta .* u_theta ./ r_mat ) ;
                        F_theta = u_theta .* ( h .* u_r ./ r_mat + d_dr_of_hu_r + d_dtheta_of_hu_theta ./ r_mat ) ...
                                + h .* ( u_r .* d_dr_of_u_theta + u_theta ./ r_mat .* d_dtheta_of_u_theta + u_theta .* u_r ./ r_mat ) ;
                % calculate some gradients of F
                        [d_dtheta_of_F_r, d_dr_of_F_r] = gradient_imp(F_r, delta_theta, delta_r) ;
                        [d_dtheta_of_F_theta, d_dr_of_F_theta] = gradient_imp(F_theta, delta_theta, delta_r) ;
                % calculate some gradients required for Laplacian(eta^2)
                        [d_dtheta_of_eta2, d_dr_of_eta2] = gradient_imp(eta.*eta, delta_theta, delta_r) ;
                        [d2_dr_dtheta_of_eta2, d2_dr2_of_eta2] = gradient_imp(d_dr_of_eta2, delta_theta, delta_r) ;
                        [d2_dtheta2_of_eta2, d2_dr_dtheta_of_eta2] = gradient_imp(d_dtheta_of_eta2, delta_theta, delta_r) ;
                % give special names to the variables we wish to retain at this timestep
                        div_F_previous = F_r ./ r_mat + d_dr_of_F_r + d_dtheta_of_F_theta ./ r_mat ;
                        k_dot_curl_F_previous = F_theta ./ r_mat + d_dr_of_F_theta - d_dtheta_of_F_r ./ r_mat ;
                        del2_eta2_previous = d2_dr2_of_eta2 + d_dr_of_eta2./r_mat + d2_dtheta2_of_eta2./r_mat./r_mat ;
                % CURRENT TIME
                % read in data
                        psi_primed = diagnostic_read(psi_filenames(i, :)) ;
                psi_primed_filteredx3 = filter(ones(1, windowSize)/windowSize, 1, [psi_primed, psi_primed, psi_primed], [], 2) ;
                psi_primed = psi_primed_filteredx3(:, N_azim+1+floor(windowSize/2) : 2*N_azim+floor(windowSize/2), :) ;
                % calculate u_r and u_theta in layer 2
                        [dpsi_dtheta, dpsi_dr] = gradient_imp(psi_primed(:, :, 2) + psi_bar(:, :, 2), delta_theta, delta_r) ;
                        u_r = -dpsi_dtheta./r_mat ;
                        u_theta = dpsi_dr ;
                % calculate h
                        eta = eta_bar + f/g_primed*(psi_primed(:, :, 2) - psi_primed(:, :, 1)) ;
                        eta = eta - mean(mean(eta .* r_mat .* area_factor)) / mean(mean(r_mat .* area_factor)) ;      % subtract <eta> to ensure volume conservation
                        h = H + eta ;
                % calculate some gradients which are needed to calculate F
                        [useless, d_dr_of_hu_r] = gradient_imp(h.*u_r, delta_theta, delta_r) ;
                        [d_dtheta_of_hu_theta, useless] = gradient_imp(h.*u_theta, delta_theta, delta_r) ;
                        [d_dtheta_of_u_r, d_dr_of_u_r] = gradient_imp(u_r, delta_theta, delta_r) ;
                        [d_dtheta_of_u_theta, d_dr_of_u_theta] = gradient_imp(u_theta, delta_theta, delta_r) ;
                % calculate components of F
                        F_r = u_r .* ( h .* u_r ./ r_mat + d_dr_of_hu_r + d_dtheta_of_hu_theta ./ r_mat ) ...
                                + h .* ( u_r .* d_dr_of_u_r + u_theta ./ r_mat .* d_dtheta_of_u_r - u_theta .* u_theta ./ r_mat ) ;
                        F_theta = u_theta .* ( h .* u_r ./ r_mat + d_dr_of_hu_r + d_dtheta_of_hu_theta ./ r_mat ) ...
                                + h .* ( u_r .* d_dr_of_u_theta + u_theta ./ r_mat .* d_dtheta_of_u_theta + u_theta .* u_r ./ r_mat ) ;
                % calculate some gradients of F
                        [d_dtheta_of_F_r, d_dr_of_F_r] = gradient_imp(F_r, delta_theta, delta_r) ;
                        [d_dtheta_of_F_theta, d_dr_of_F_theta] = gradient_imp(F_theta, delta_theta, delta_r) ;
                % calculate some gradients required for Laplacian(eta^2)
                        [d_dtheta_of_eta2, d_dr_of_eta2] = gradient_imp(eta.*eta, delta_theta, delta_r) ;
                        [d2_dr_dtheta_of_eta2, d2_dr2_of_eta2] = gradient_imp(d_dr_of_eta2, delta_theta, delta_r) ;
                        [d2_dtheta2_of_eta2, d2_dr_dtheta_of_eta2] = gradient_imp(d_dtheta_of_eta2, delta_theta, delta_r) ;
                % give special names to the variables we wish to retain at this timestep
                        div_F_current = F_r ./ r_mat + d_dr_of_F_r + d_dtheta_of_F_theta ./ r_mat ;
                        k_dot_curl_F_current = F_theta ./ r_mat + d_dr_of_F_theta - d_dtheta_of_F_r ./ r_mat ;
                        del2_eta2_current = d2_dr2_of_eta2 + d_dr_of_eta2./r_mat + d2_dtheta2_of_eta2./r_mat./r_mat ;
                % NEXT TIME
                % read in data
                        if i == length(inds)
                                tmp_PSI_ = ['PSI_.0000000000'] ;
                                tmp_PSI_(16-length(num2str(finish+increment)) : 15) = num2str(finish+increment) ;
                                psi_primed = diagnostic_read(tmp_PSI_) ;
                        else
                                psi_primed = diagnostic_read(psi_filenames(i+1, :)) ;
                        end
                psi_primed_filteredx3 = filter(ones(1, windowSize)/windowSize, 1, [psi_primed, psi_primed, psi_primed], [], 2) ;
                psi_primed = psi_primed_filteredx3(:, N_azim+1+floor(windowSize/2) : 2*N_azim+floor(windowSize/2), :) ;
                % calculate u_r and u_theta in layer 2
                        [dpsi_dtheta, dpsi_dr] = gradient_imp(psi_primed(:, :, 2) + psi_bar(:, :, 2), delta_theta, delta_r) ;
                        u_r = -dpsi_dtheta./r_mat ;
                        u_theta = dpsi_dr ;
                % calculate h
                        eta = eta_bar + f/g_primed*(psi_primed(:, :, 2) - psi_primed(:, :, 1)) ;
                        eta = eta - mean(mean(eta .* r_mat .* area_factor)) / mean(mean(r_mat .* area_factor)) ;      % subtract <eta> to ensure volume conservation
                        h = H + eta ;
                % calculate some gradients which are needed to calculate F
                        [useless, d_dr_of_hu_r] = gradient_imp(h.*u_r, delta_theta, delta_r) ;
                        [d_dtheta_of_hu_theta, useless] = gradient_imp(h.*u_theta, delta_theta, delta_r) ;
                        [d_dtheta_of_u_r, d_dr_of_u_r] = gradient_imp(u_r, delta_theta, delta_r) ;
                        [d_dtheta_of_u_theta, d_dr_of_u_theta] = gradient_imp(u_theta, delta_theta, delta_r) ;
                % calculate components of F 
                        F_r = u_r .* ( h .* u_r ./ r_mat + d_dr_of_hu_r + d_dtheta_of_hu_theta ./ r_mat ) ...
                                + h .* ( u_r .* d_dr_of_u_r + u_theta ./ r_mat .* d_dtheta_of_u_r - u_theta .* u_theta ./ r_mat ) ;
                        F_theta = u_theta .* ( h .* u_r ./ r_mat + d_dr_of_hu_r + d_dtheta_of_hu_theta ./ r_mat ) ...
                                + h .* ( u_r .* d_dr_of_u_theta + u_theta ./ r_mat .* d_dtheta_of_u_theta + u_theta .* u_r ./ r_mat ) ;
                % calculate some gradients of F
                        [d_dtheta_of_F_r, d_dr_of_F_r] = gradient_imp(F_r, delta_theta, delta_r) ;
                        [d_dtheta_of_F_theta, d_dr_of_F_theta] = gradient_imp(F_theta, delta_theta, delta_r) ;
                % calculate some gradients required for Laplacian(eta^2)
                        [d_dtheta_of_eta2, d_dr_of_eta2] = gradient_imp(eta.*eta, delta_theta, delta_r) ;
                        [d2_dr_dtheta_of_eta2, d2_dr2_of_eta2] = gradient_imp(d_dr_of_eta2, delta_theta, delta_r) ;
                        [d2_dtheta2_of_eta2, d2_dr_dtheta_of_eta2] = gradient_imp(d_dtheta_of_eta2, delta_theta, delta_r) ;
                % give special names to the variables we wish to retain at this timestep
                        div_F_next = F_r ./ r_mat + d_dr_of_F_r + d_dtheta_of_F_theta ./ r_mat ;
                        k_dot_curl_F_next = F_theta ./ r_mat + d_dr_of_F_theta - d_dtheta_of_F_r ./ r_mat ;
                        del2_eta2_next = d2_dr2_of_eta2 + d_dr_of_eta2./r_mat + d2_dtheta2_of_eta2./r_mat./r_mat ;
                % calculate Lighthill radiation term
                LRT = ( div_F_next - div_F_previous ) / (2*delta_t*increment) ...
                        + f * k_dot_curl_F_current ...
                        - g_primed/4 * (del2_eta2_next - del2_eta2_previous) / (2*delta_t*increment) ;
                % plot Lighthill radiation term
                LRT(:, N_azim+1) = LRT(:, 1) ;                          % to join up data across final azimuthal gridpoint
                contourf(x_mat(:,:), y_mat(:,:), abs(LRT(:, :)), 20) ;
                shading flat
                colorbar_handle = colorbar ;
                set(get(colorbar_handle,'YLabel'),'String', ...
                        ['|LRT| (r, \theta, t=', num2str(inds(i)*delta_t), 's)   (m s^-^3)']) ;
                title([wd, '   (F, d) = (', num2str(F), ', ', num2str(d), ')']) ;
                axis([-1.1*b, 1.1*b, -1.1*b, 1.1*b])
                xlabel('x (m)') ;
                ylabel('y (m)') ;
                axis equal
		axis([-1.1*b, 1.1*b, -1.1*b, 1.1*b])
                grid on

		pause(0)

        end

elseif option == 29 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

	% contour plot of horizontal divergence

	psi_bar = zeros(N_rad, N_azim, N_layer) ;
	for i = 1 : N_rad
		for k = 1 : N_layer
			psi_bar(i, :, k) = 0.5*delta_omega(k)*r(i)*r(i) ;
		end
	end

	eta_bar = zeros(N_rad, N_azim) ;
	for i = 1 : N_rad
		eta_bar(i, :) = omega*omega*r(i)*r(i)*(1.0 - (lid_delta_omega/omega)/(delta_rho/rho_bar))/(2.0*g) ;
	end
                
	x_mat(:, N_azim+1) = x_mat(:, 1) ;				% } to join up data across final azimuthal gridpoint
	y_mat(:, N_azim+1) = y_mat(:, 1) ;				% }

	for i = 1 : length(inds)

		% get eta at time i-1
		if i == 1
			tmp_PSI_ = ['PSI_.0000000000'] ;
			tmp_PSI_(16-length(num2str(start-increment)) : 15) = num2str(start-increment) ;
			psi_primed = diagnostic_read(tmp_PSI_) ;
		else
			psi_primed = diagnostic_read(psi_filenames(i-1, :)) ;
		end
		eta_minus = eta_bar + f/g_primed*(psi_primed(:, :, 2) - psi_primed(:, :, 1)) ;
		eta_minus = eta_minus - mean(mean(eta_minus .* r_mat .* area_factor)) / mean(mean(r_mat .* area_factor)) ;	% volume cons.

		% get eta at time i
		psi_primed = diagnostic_read(psi_filenames(i, :)) ;
                eta_now = eta_bar + f/g_primed*(psi_primed(:, :, 2) - psi_primed(:, :, 1)) ;
                eta_now = eta_now - mean(mean(eta_now .* r_mat .* area_factor)) / mean(mean(r_mat .* area_factor)) ;		% volume cons. 
		[dpsi_dtheta_now, dpsi_dr_now] = gradient_imp(psi_primed(:, :, layer) + psi_bar(:, :, layer), delta_theta, delta_r) ;

		% get eta at time i+1
		if i == length(inds)
			tmp_PSI_ = ['PSI_.0000000000'] ;
			tmp_PSI_(16-length(num2str(finish+increment)) : 15) = num2str(finish+increment) ;
			psi_primed = diagnostic_read(tmp_PSI_) ;
		else
			psi_primed = diagnostic_read(psi_filenames(i+1, :)) ;
		end
                eta_plus = eta_bar + f/g_primed*(psi_primed(:, :, 2) - psi_primed(:, :, 1)) ;
                eta_plus = eta_plus - mean(mean(eta_plus .* r_mat .* area_factor)) / mean(mean(r_mat .* area_factor)) ;		% volume cons. 

		% calculate delta using D/Dt = d/dt + u.grad
		derivative = (eta_plus - eta_minus) / (2*delta_t*increment) ;				% local rate of change
		[deta_dtheta_now, deta_dr_now] = gradient_imp(eta_now, delta_theta, delta_r) ;
		derivative = derivative + ( - dpsi_dtheta_now .* deta_dr_now + dpsi_dr_now .* deta_dtheta_now ) ./ r_mat ;	% advection in a layer
		if (layer == 1)
			delta = derivative ./ (H - eta_now);
		elseif (layer == 2)
			delta = - derivative ./ (H + eta_now);
		end

		% filter to remove spurious noise
                deltax3 = filter(ones(1, windowSize)/windowSize, 1, [delta, delta, delta], [], 2) ;
                delta = deltax3(:, N_azim+1+floor(windowSize/2) : 2*N_azim+floor(windowSize/2)) ;

		% plot delta
		delta(:, N_azim+1) = delta(:, 1) ;
		contourf(x_mat(:,:), y_mat(:,:), delta(:, :), 20) ;
		shading flat
		colorbar_handle = colorbar ;
		set(get(colorbar_handle,'YLabel'),'String', ...
			['\delta (r, \theta, layer=', num2str(layer), ', t=', num2str(inds(i)*delta_t), 's)   (s^-^1)']) ;
		title([wd, '   (F, d) = (', num2str(F), ', ', num2str(d), ')']) ;
		axis([-1.1*b, 1.1*b, -1.1*b, 1.1*b])
		xlabel('x (m)') ;
		ylabel('y (m)') ;
		axis equal
		grid on
		pause(0)

		if (iteration ~= 0)
			M(i) = getframe;
		end
                
	end

	if (iteration ~= 0)
		movie(M, iteration, fps)
	end

elseif option == 30 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                        
        % mid-radius interface height Hovmueller diagram

        eta_bar = zeros(N_rad, N_azim) ;
        for i = 1 : N_rad
                eta_bar(i, :) = omega*omega*r(i)*r(i)*(1.0 - (lid_delta_omega/omega)/(delta_rho/rho_bar))/(2.0*g) ;
        end

        h = [] ;
        for i = 1 : length(inds)
		psi_primed = diagnostic_read(psi_filenames(i, :)) ;
		eta_primed = f/g_primed*(psi_primed(:, :, 2) - psi_primed(:, :, 1)) ;
		eta = eta_primed + eta_bar ;
		eta = eta - mean(mean(eta.* r_mat .* area_factor)) / mean(mean(r_mat .* area_factor));
		h = [h ; H+eta(floor(N_rad/2), :)] ;
	end

        h(:,N_azim+1) = h(:,1) ;
        theta(N_azim+1) = 2*pi ;
        contourf(inds*delta_t, theta*180/pi, 1000*h(:, :)', 20) ;
        set(gca, 'Box', 'on') ;
        shading flat
        colorbar_handle = colorbar ;
        set(get(colorbar_handle,'YLabel'),'String', ...
                ['h (r=mid-radius, \theta, t)   (mm)']) ;
        title([wd, '   (F, d) = (', num2str(F), ', ', num2str(d), ')']) ;
        xlabel('t (s)') ;
        ylabel('\theta (degrees)') ;
        set(gca, 'YTick', 0:60:360) ;
	set(gcf, 'PaperPosition', [0.25 2.5 16 6]) ;    % use a font size of 20

elseif option == 31 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                
        % zonally-averaged zonal velocity as a function of radius
                
        psi_bar = zeros(N_rad, N_azim, N_layer) ;
        for i = 1 : N_rad 
                for k = 1 : N_layer
                        psi_bar(i, :, k) = 0.5*delta_omega(k)*r(i)*r(i) ;
                end
        end
                        
        for i = 1 : length(inds)
           
                psi_primed = diagnostic_read(psi_filenames(i, :)) ;
        
                [dpsi_dtheta, dpsi_dr] = gradient_imp(psi_primed(:, :, layer) + psi_bar(:, :, layer), delta_theta, delta_r) ;
                u_theta = dpsi_dr ;
		u_theta_mean = mean(u_theta, 2) ;

		plot(r, u_theta_mean*1000) ;
                title([wd, '   (F, d) = (', num2str(F), ', ', num2str(d), ')']) ;
                xlabel('r (m)') ;
		ylabel(['zonally-averaged u_\theta (r, layer=', num2str(layer), ', t=', num2str(inds(i)*delta_t), 's)   (mm s^{-1})']) ;
                grid on
                pause(0)
                
                if (iteration ~= 0)
                        M(i) = getframe;
                end
           
        end

        if (iteration ~= 0)
                movie(M, iteration, fps)
        end

elseif option == 32 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

        % time-radius Hovmueller plot of zonally-averaged zonal velocity
                
        psi_bar = zeros(N_rad, N_azim, N_layer) ;
        for i = 1 : N_rad
                for k = 1 : N_layer
                        psi_bar(i, :, k) = 0.5*delta_omega(k)*r(i)*r(i) ;
                end
        end     

	u_theta_mean_hov = [];        
        for i = 1 : length(inds)
           
                psi_primed = diagnostic_read(psi_filenames(i, :)) ;
                [dpsi_dtheta, dpsi_dr] = gradient_imp(psi_primed(:, :, layer) + psi_bar(:, :, layer), delta_theta, delta_r) ;
                u_theta = dpsi_dr ;
                u_theta_mean = mean(u_theta, 2) ;
		u_theta_mean_hov = [u_theta_mean_hov , u_theta_mean] ;

	end
        
        contourf(inds*delta_t, r, u_theta_mean_hov*1000, 20) ;
        shading flat
        colorbar_handle = colorbar ;
        set(get(colorbar_handle,'YLabel'),'String', ...
            ['zonally-averaged u_\theta (r, layer=', num2str(layer), ', t)   (mm s^{-1})']) ;
        title([wd, '   (F, d) = (', num2str(F), ', ', num2str(d), ')']) ;
        xlabel('t (s)') ;
	ylabel('r (m)') ;
        grid on
        pause(0)
                
else %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

	fprintf(1, '     ERROR!  Option number must be an integer.\n\n') ;

end %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

fprintf(1, '\n') ;

if option ~= 0
	pause
end

end

return

