!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
! init_model.f90	Initialize the model by allocating and evaluating those global
!			arrays, and evaluating those global parameters, defined in the 
!			phys_params and grid_params modules.
!
!			Called by PROGRAM qgam.
! 
! twnh 14Aug01 & pdw 26Mar02
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

SUBROUTINE init_model

! NAG routines used
USE nag_math_constants, ONLY : nag_pi

! load global variables
USE grid_params
USE phys_params

IMPLICIT NONE

! declaration of local variables
REAL (KIND=wp), DIMENSION(100)	::  rho, new			! } physical parameters used to derive global variables but not needed
REAL (KIND=wp)			::  b, S, delta_m, chi_t	! } globally themselves; maximum number of layers = 100 is hard-coded here
INTEGER				::  i, ii, j			! indices

! external parameter file
NAMELIST /grid_definition/	N_rad, N_azim, N_layer
NAMELIST /time_stepping/	delta_t, start_step, end_step, Robert_filter_parameter, dump_period, pickup_dump_period, debug
NAMELIST /system_dimensions/	a, b, H, s_top, s_bot
NAMELIST /fluid_properties/	rho, S, new
NAMELIST /forcing_dissipation/	initial_amplitude, omega, lid_delta_omega, g, new_hyper, relax_rate, relax_type, reset_period, &
				internal_ekman, noise_amp, d_dt_noise_amp

WRITE(*,*) ' --------------------------------------------------------------------------------------------------------------------'
WRITE(*,*) ' init_model: Initializing internal parameters from file: qgam.dat.'
WRITE(*,*)

pi = nag_pi(1.0_wp)

new = 0.0_wp
rho = 0.0_wp

! open external parameter file
OPEN(UNIT=10, FILE='qgam.dat')

! grid definition
READ(10, NML=grid_definition)
WRITE(*,*) '  Grid definition:'
WRITE(*,*) '  Grid points: (radius, azimuth, layers) : [',N_rad,', ',N_azim,', ',N_layer,'].'
WRITE(*,*) 

! time stepping
READ(10, NML=time_stepping)
WRITE(*,*) '  Time stepping: from step [',start_step,'] to [',end_step,'] in steps of [',delta_t,'s].'
WRITE(*,*) '  Robert filter parameter is [',Robert_filter_parameter,'].'
WRITE(*,*) '  Dump to disk every [',dump_period,'] steps.'
WRITE(*,*) '  Pickup dump to disk every [',pickup_dump_period,'] steps.'
IF (debug == 1) THEN
	WRITE(*,*) '  Debug messages are ON.'
ELSE
	WRITE(*,*) '  Debug messages are OFF.'
ENDIF
WRITE(*,*) 

! system dimensions
WRITE(*,*) '  System dimensions:'
READ(10, NML=system_dimensions)
WRITE(*,*) '  Layer depth [',H,'m].'
WRITE(*,*) '  Radius of inner boundary [',a,'m].'
WRITE(*,*) '  Radius of outer boundary [',b,'m].'
WRITE(*,*) '  Radial slope of base [',s_bot,'].'
WRITE(*,*) '  Radial slope of lid [',s_top,'].'
WRITE(*,*)

! fluid properties
WRITE(*,*) '  Fluid properties:'
READ(10, NML=fluid_properties)
DO ii = 1, N_layer
	WRITE(*,*) '  Layer ',ii,' density [',rho(ii),'kg m^-3].'
END DO
WRITE(*,*) '  Interfacial tension [',S,'N m^-1].'
DO ii = 1, N_layer
	WRITE(*,*) '  Layer ',ii,' kinematic viscosity [',new(ii),'m^2 s^-1].'
END DO
WRITE(*,*) 

! forcing and dissipation
WRITE(*,*) '  Forcing/dissipation:'
READ(10, NML=forcing_dissipation)
WRITE(*,*) '  Amplitude of initial PV perturbation [',initial_amplitude,'s^-1].'
WRITE(*,*) '  Rotation rate [',omega,'rad s^-1].'
WRITE(*,*) '  Gravitational acceleration [',g,'m s^-2].'
WRITE(*,*) '  2nd order PV hyperdiffusion coefficient [',new_hyper,'m^2 s^-1].'
WRITE(*,*) '  Rotating upper lid with differential rotation rate [',lid_delta_omega,'rad s^-1].'
IF (relax_type == 1) THEN
	WRITE(*,*) '  Relaxation to specified streamfunction is on.'
	WRITE(*,*) '  No explicit relaxation to specified QGPV.'
ELSEIF (relax_type == 2) THEN
	WRITE(*,*) '  No explicit relaxation to specified streamfunction.'
	WRITE(*,*) '  Relaxation to specified QGPV is on at rate [',relax_rate,'s^-1].'
ELSEIF (relax_type == 3) THEN
	WRITE(*,*) '  Relaxation to specified streamfunction is on.'
	WRITE(*,*) '  Relaxation to specified QGPV is on at rate [',relax_rate,'s^-1].'
ELSE
	WRITE(*,*) '  No explicit relaxation to specified streamfunction or QGPV.'
ENDIF
WRITE(*,*) '  Mean PPV is reset to zero every [',reset_period,'] steps.'
IF (internal_ekman == 1) THEN
	WRITE(*,*) '  Internal Ekman layers are ON.'
ELSE
	WRITE(*,*) '  Internal Ekman layers are OFF.'
ENDIF
WRITE(*,*) '  Stochastic forcing amplitude (initial value) [',noise_amp,'s^-2].'
WRITE(*,*) '  Stochastic forcing amplitude (rate of change) [',d_dt_noise_amp,'s^-3].'
WRITE(*,*)

! derived parameters
prev_step = 1
this_step = 2
next_step = 3
f = 2.0_wp*omega							! Coriolis parameter (s^-1)
g_primed = 2.0_wp*g*(rho(2) - rho(1))/(rho(2) + rho(1))			! reduced gravity (m s^-2)
delta_m = SQRT(S/(g*(rho(2) - rho(1))))					! characteristic static meniscus width (m)
itcc = 1.0_wp/(1.0_wp - (2.0_wp*f*f*delta_m*delta_m/(g_primed*H)))	! interfacial tension correction coefficient
lambda_bt = 0.0_wp							! eigenvalue in barotropic Helmholtz equation
lambda_bc = itcc*2.0_wp*f*f/(g_primed*H)				! eigenvalue in baroclinic Helmholtz equation
delta_r = (b - a)/(REAL(N_rad-1, KIND=wp))				! radial grid point separation (m)
delta_theta = 2.0_wp*pi/(REAL(N_azim, KIND=wp))				! azimuthal grid point separation (rad)
forcing_coeff = f*f*(omega/g - lid_delta_omega/g_primed)/(2.0_wp*H)	! coefficient for forcing by equilibrium flow

! derived arrays
ALLOCATE(	spindown_rate(N_layer),			&		! Ekman spindown rate for each layer (s^-1)
		chi(N_layer),				&		! dimensionless viscosity factor for each layer
		delta_omega(N_layer),			&		! solid body rotation rate for each layer (rad s^-1)
		rad_dist(N_rad),			&		! radial positions of grid points (m)
		area_element(N_rad, N_azim)		)		! area of each grid-box (m^2)
spindown_rate = SQRT(omega*new)/H
chi = 0.0_wp
IF (internal_ekman /= 1) THEN
        spindown_rate(2:N_layer-1) = 0.0_wp                             ! no internal Ekman layers
ELSEIF (N_layer == 2) THEN                                              ! set variables for 2 layers only
        chi(1) = SQRT(new(1))/(SQRT(new(1)) + SQRT(new(2)))
        chi(2) = SQRT(new(2))/(SQRT(new(1)) + SQRT(new(2)))
        chi_t = SQRT(new(2)/new(1))
        delta_omega(1) = (2.0_wp + chi_t)*lid_delta_omega/(2.0_wp*(1 + chi_t))
        delta_omega(2) = lid_delta_omega/(2.0_wp*(1 + chi_t))
ELSE 
        WRITE(*,*) '   ERROR!  Internal Ekman layers are currently only implemented for 2 layers.'
        STOP
ENDIF
DO i=1, N_rad
       rad_dist(i) = a + delta_r*REAL(i-1, KIND=wp)
END DO
DO i=1, N_rad
	DO j=1, N_azim
		IF (i==1) THEN
			area_element(i, j) = 0.5_wp*rad_dist(i)*delta_theta*delta_r
		ELSEIF (i==N_rad) THEN
			area_element(i, j) = 0.5_wp*rad_dist(i)*delta_theta*delta_r
		ELSE
			area_element(i, j) = rad_dist(i)*delta_theta*delta_r
		ENDIF
	END DO
END DO
area = SUM(area_element)
WRITE(*,*) '  sum of area elements is [', area, 'm^2].'
WRITE(*,*) '  pi*(b^2-a^2)         is [', pi*(rad_dist(N_rad)**2-rad_dist(1)**2), 'm^2].'
WRITE(*,*)

! diagnostic quantities for user
WRITE(*,*) '  Non-dimensional parameters:'
WRITE(*,*) '  Internal Froude number [',(f*f*(b-a)*(b-a))/(g_primed*H),'].'
IF( lid_delta_omega > 1.0e-4) THEN
	WRITE(*,*) '  Dissipation parameter (mean) [',(SQRT(omega*(new(1)+new(2))*0.5_wp))/(H*lid_delta_omega),'].'
	DO i=1, N_layer
		WRITE(*,*) '  Dissipation parameter (layer',i,') [',(SQRT(omega*new(i)))/(H*lid_delta_omega),'].'
	END DO
	WRITE(*,*) '  Azimuthal Courant number [',(delta_t*lid_delta_omega)/(2.0_wp*delta_theta),'].'
	WRITE(*,*) '  Richardson number at r=b [',4.0_wp*g_primed*H/(b*lid_delta_omega)**2,'].'
ENDIF
WRITE(*,*) '  Rossby number [',0.5_wp*lid_delta_omega/omega,'].'
WRITE(*,*) '  Interfacial tension number [',S/(g*(rho(2)-rho(1))*(b-a)*(b-a)),'].'
WRITE(*,*) '  Ekman number (mean) [',(new(1)+new(2))/(4.0_wp*omega*(b-a)*(b-a)),'].'
WRITE(*,*) '  External Froude number [',(f*f*(b-a)*(b-a))/(8.0_wp*g*H),'].'
WRITE(*,*) '  Interfacial tension correction coefficient [',itcc,'].'
WRITE(*,*)
WRITE(*,*) '  Memory needed to store single precision PSI_ and QGPV files [',8.0_wp*N_rad*N_azim*N_layer*end_step/(1024.0_wp**2)/ &
		REAL(dump_period, KIND=wp), 'MB].'
WRITE(*,*) ' --------------------------------------------------------------------------------------------------------------------'
WRITE(*,*)
WRITE(*,*)

END SUBROUTINE init_model
