!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
! jacobian.f90	Jacobian in cylindrical polar co-ordinates for advection.
!		Subroutine calculates:
!					tend = jacobian (q_in, psi_in)
!					     = - (1/r) J_{r, theta) (psi_in, q_in).
!		Use Arakawa Jacobian which guarantees undamaged conservation properties.
!		Modify Cartesian form to yield suitable form in cylindrical coordinates.
!		Written to minimise the number of multiplications (as these are expensive).
!
!		Called by PROGRAM qgam.
! 
! twnh 14Aug01
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

SUBROUTINE jacobian(q_in, psi_in, tend)

! load global variables
USE grid_params

IMPLICIT NONE

! declaration of local variables
REAL (KIND=wp), DIMENSION(N_rad, N_azim, N_layer), INTENT(IN)		:: q_in, psi_in			! input fields
REAL (KIND=wp), DIMENSION(N_rad, N_azim, N_layer), INTENT(INOUT)	:: tend				! tendency due to advection
INTEGER									:: i, j, ip, im, jp, jm		! indices
REAL (KIND=wp)								:: fac				! temporary variable

WRITE(*,*) ' jacobian...'

! calculation
DO i = 1, N_rad												! loop over radial direction
	ip = i+1
	im = i-1
	fac = 1.0_wp/(12.0_wp*delta_r*delta_theta*rad_dist(i))						! normalization factor for this radius
	DO j = 1, N_azim										! loop over azimuthal direction
		jp = j+1
		jm = j-1
		IF (j == 1) THEN									! azimuthal periodicity
			jm = N_azim
		ELSEIF (j == N_azim) THEN 
			jp = 1
		ENDIF

		IF (i == 1) THEN									! innermost boundary: "left" case
 			tend(i,j,:) = tend(i,j,:) - 2.0_wp * (										&
			(psi_in(i ,jm,:) + psi_in(ip,jm,:) - psi_in(i ,jp,:) - psi_in(ip,jp,:)) * (q_in(ip,j ,:) + q_in(i ,j ,:)) +	&
			(psi_in(ip,j ,:) + psi_in(ip,jp,:) - psi_in(i ,j ,:) - psi_in(i ,jp,:)) * (q_in(i ,jp,:) + q_in(i ,j ,:)) -	&
			(psi_in(ip,jm,:) + psi_in(ip,j ,:) - psi_in(i ,jm,:) - psi_in(i ,j ,:)) * (q_in(i ,j ,:) + q_in(i ,jm,:)) +	&
							(psi_in(ip,j ,:) - psi_in(i ,jp,:)) * (q_in(ip,jp,:) + q_in(i ,j ,:)) -		&
							(psi_in(ip,j ,:) - psi_in(i ,jm,:)) * (q_in(i ,j ,:) + q_in(ip,jm,:))		)

		ELSEIF (i == N_rad) THEN								! outermost boundary: "right" case
			tend(i,j,:) = tend(i,j,:) - 2.0_wp * (										&
			(psi_in(im,jm,:) + psi_in(im,j ,:) - psi_in(i ,jm,:) - psi_in(i ,j ,:)) * (q_in(i ,j ,:) + q_in(i ,jm,:)) -	&
			(psi_in(im,jm,:) + psi_in(i ,jm,:) - psi_in(im,jp,:) - psi_in(i ,jp,:)) * (q_in(i ,j ,:) + q_in(im,j ,:)) -	&
			(psi_in(im,j ,:) + psi_in(im,jp,:) - psi_in(i ,j ,:) - psi_in(i ,jp,:)) * (q_in(i ,j ,:) + q_in(i,jp ,:)) -	&
							(psi_in(i ,jm,:) - psi_in(im,j ,:)) * (q_in(i ,j ,:) + q_in(im,jm,:)) +		&
							(psi_in(i ,jp,:) - psi_in(im,j ,:)) * (q_in(im,jp,:) + q_in(i ,j ,:))		)

		ELSE 											! interior point
			tend(i,j,:) = tend(i,j,:) - (											&
			(psi_in(i ,jm,:) + psi_in(ip,jm,:) - psi_in(i ,jp,:) - psi_in(ip,jp,:)) * (q_in(ip,j ,:) + q_in(i ,j ,:)) - 	&
			(psi_in(im,jm,:) + psi_in(i ,jm,:) - psi_in(im,jp,:) - psi_in(i ,jp,:)) * (q_in(i ,j ,:) + q_in(im,j ,:)) +	&
			(psi_in(ip,j ,:) + psi_in(ip,jp,:) - psi_in(im,j ,:) - psi_in(im,jp,:)) * (q_in(i ,jp,:) + q_in(i ,j ,:)) -	&
			(psi_in(ip,jm,:) + psi_in(ip,j ,:) - psi_in(im,jm,:) - psi_in(im,j ,:)) * (q_in(i ,j ,:) + q_in(i ,jm,:)) +	&
							(psi_in(ip,j ,:) - psi_in(i ,jp,:)) * (q_in(ip,jp,:) + q_in(i ,j ,:)) -		&
							(psi_in(i ,jm,:) - psi_in(im,j ,:)) * (q_in(i ,j ,:) + q_in(im,jm,:)) +		&
							(psi_in(i ,jp,:) - psi_in(im,j ,:)) * (q_in(im,jp,:) + q_in(i ,j ,:)) -		&
							(psi_in(ip,j ,:) - psi_in(i ,jm,:)) * (q_in(i ,j ,:) + q_in(ip,jm,:))		)
		ENDIF

	END DO												! end azimthual loop
	tend(i,:,:) = tend(i,:,:)*fac									! scale by grid factor
END DO													! end radial loop

END SUBROUTINE jacobian
