!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
! main.f90	Highest-level routine in quasi-geostrophic annulus model.
!		q   (1:N_rad, 1:N_azim, 1:N_layer, 1:3) is perturbation QGPV field (s^-1) at three consecutive time slices.
!		psi (1:N_rad, 1:N_azim, 1:N_layer, 1:3) is perturbation streamfunction field (m^2 s^-1) at three consecutive time slices.
!		These fields are perturbed away from an equilibrium state with each layer in solid-body rotation.
!
!		Model code written by Thomas W N Haine and Paul D Williams,
!		with advice from Peter L Read, Stephen R Lewis and Yasuhiro Yamazaki.
!
! twnh 13Aug01
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!


PROGRAM qgam

! load global variables
USE grid_params
USE dyn_vars

IMPLICIT NONE

! declaration of local variables
INTEGER				:: tmp_step, k			! indices

WRITE(*,*) ' main: QUASI-GEOSTROPHIC ANNULUS MODEL'
WRITE(*,*)

! initialize
CALL init_model							! set up model variables
CALL init_solver						! set up solver arrays
CALL init_state							! set up initial state fields

DO step = start_step, end_step					! time-stepping loop	

	WRITE(*,*) 
	WRITE(*,*) ' main: step',step

	! update time stepping indices
	tmp_step  = prev_step
	prev_step = this_step
	this_step = next_step
	next_step = tmp_step

	! reset tendency to zero
	q_tend = 0.0_wp

	! advection routine - increments PV tendency (conserves mean PV)
	CALL jacobian(q(:,:,:,this_step), psi(:,:,:,this_step), q_tend)

	IF (debug == 1) THEN
		DO k = 1, N_layer
			WRITE(*,*) '     mean PV tendency in layer',k,'is [ ',SUM(area_element*q_tend(:,:,k))/area,'s^-2 ].'
		END DO
	ENDIF

	! forcing routine - increments PV tendency (conserves mean PV)
	CALL forcing(q(:,:,:,this_step), psi(:,:,:,this_step), q_tend)

	IF (debug == 1) THEN
		DO k = 1, N_layer
			WRITE(*,*) '     mean PV tendency in layer',k,'is [ ',SUM(area_element*q_tend(:,:,k))/area,'s^-2 ].'
		END DO
	ENDIF

	! dissipation routine - increments PV tendency (slight non-conservation of mean PV because of hyperdiffusion)
	CALL dissipation(psi(:,:,:,prev_step), q(:,:,:,prev_step), q_tend)		! uses prev_step, i.e. time-tagged diffusion

	IF (debug == 1) THEN
		DO k = 1, N_layer
			WRITE(*,*) '     mean PV tendency in layer',k,'is [ ',SUM(area_element*q_tend(:,:,k))/area,'s^-2 ].'
		END DO
	ENDIF

	! update PV field using tendency
	CALL step_q(q(:,:,:,prev_step), q(:,:,:,this_step), q(:,:,:,next_step), q_tend)

	IF (debug == 1) THEN
		DO k = 1, N_layer
			WRITE(*,*) '     mean PV in layer',k,'is          [ ',SUM(area_element*q(:,:,k,next_step))/area,'s^-1 ].'
		END DO
	ENDIF

	! reset mean PPV to zero?
	IF (MODULO(step, reset_period) == 0) THEN
		WRITE(*,*) ' resetting mean PV to zero in each layer...'
		DO k=1, N_layer
			q(:,:,k,next_step) = q(:,:,k,next_step) - SUM(area_element*q(:,:,k,next_step))/area
		END DO
	ENDIF

	IF (debug == 1 .AND. MODULO(step, reset_period) == 0) THEN
		DO k = 1, N_layer
			WRITE(*,*) '     mean PV in layer',k,'is          [ ',SUM(area_element*q(:,:,k,next_step))/area,'s^-1 ].'
		END DO
	ENDIF

	! Helmholtz solver to invert q field for psi field
	CALL solver(q(:,:,:,next_step), psi(:,:,:,next_step))

	IF (debug == 1) THEN
		WRITE(*,*) '     PSI (1,             1:3,  1)  : [', (psi(1,k,1,next_step), k=1,3), 'm^2 s^-1 ].'
		WRITE(*,*) '     PSI (N_rad,         1:3,  1)  : [', (psi(N_rad,k,1,next_step), k=1,3), 'm^2 s^-1 ].'
		WRITE(*,*) '     PSI (1,             1:3,  2)  : [', (psi(1,k,2,next_step), k=1,3), 'm^2 s^-1 ].'
		WRITE(*,*) '     PSI (N_rad,         1:3,  2)  : [', (psi(N_rad,k,2,next_step), k=1,3), 'm^2 s^-1 ].'
		WRITE(*,*) '     PSI (1:3,           mean, 1)  : [', &
					(SUM(psi(k,:,1,next_step))/REAL(N_azim,KIND=wp), k=1,3), 'm^2 s^-1 ].'
		WRITE(*,*) '     PSI (N_rad-2:N_rad, mean, 1)  : [', &
					(SUM(psi(k,:,1,next_step))/REAL(N_azim,KIND=wp), k=N_rad-2,N_rad), 'm^2 s^-1 ].'
		WRITE(*,*) '     PSI (1:3,           mean, 2)  : [', &
					(SUM(psi(k,:,2,next_step))/REAL(N_azim,KIND=wp), k=1,3), 'm^2 s^-1 ].'
		WRITE(*,*) '     PSI (N_rad-2:N_rad, mean, 2)  : [', &
					(SUM(psi(k,:,2,next_step))/REAL(N_azim,KIND=wp), k=N_rad-2,N_rad), 'm^2 s^-1 ].'
	ENDIF

	! save data to disk
	CALL save_fields

END DO								! time-stepping loop

WRITE(*,*)
WRITE(*,*) ' main: model integration stopping normally'

END PROGRAM qgam
